# File svdfit/esky1.cl
# February 23, 2000
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# ESKY1 -- Remove sky from spectra by fitting eigenvectors and removing the
#	  fit for a file or list of files

procedure esky1 (infile, outfile)

string	infile=""	{prompt="Input spectrum file"}
string	specnum="1"	{prompt="Range of spectra in multispec or 2D file"}
int	specband=1	{prompt="Band if multispec file"}
string	indir=""	{prompt="Input directory"}
string	outform1="twodspec" {prompt="First pass format (multispec|oned|twod)"}
string	intfile=""	{prompt="Intermediate spectrum file"}
file	vecdir="svdfit$lib/"	{prompt="Directory for eigenvectors"}
file	vecfile1="evec1.fits"	{prompt="First pass eigenvector file"}
string	vectors1="1-2"	{prompt="First pass eigenvectors"}
string	fixbad1="zero"	{prompt="Fix bad lines in first pass (zero fill no)"}
real	renorm1=1.0	{prompt="Renormalize (divide by mean) in first pass"}
int	order1=40	{prompt="First pass continuum order"}
bool	delfile=yes	{prompt="Delete intermediate files (y or n)"}
file	vecfile2="evec2.fits"	{prompt="Second pass eigenvector file"}
string	fixbad2="zero"	{prompt="Fix bad lines in second pass (zero fill no)"}
real	renorm2=0.0	{prompt="Renormalize (divide by mean) in second pass"}
string	vectors2="1-10"	{prompt="Second pass eigenvectors"}
int	order2=40	{prompt="Second pass continuum order"}
string	outform2="twodspec" {prompt="Second pass format (multispec|oned|twod)"}
file	badlines="badskynao.dat" {prompt="File with lines to eliminate"}
string	outfile=""	{prompt="Output spectrum file"}
string	outdir=""	{prompt="Output directory"}
string	outform=""	{prompt="Final spectrum format, if not outform2"}
bool	plot=yes	{prompt="Plot results on display (y or n)"}
bool	cursor=no	{prompt="Wait after plot with cursor (y or n)"}
bool	verbose=no	{prompt="Print what's happening (y or n)"}
bool	debug=no	{prompt="Debugging mode (y or n)"}

begin

bool	vb, delf, makeout, delint
int	i
file	filelist, odir, sfile
file	localfile, ifile, jfile, ofile
file	listfile, eigtemp
file	cfile, tfile
string	tchar, object, oform1, oform2, oform

	vb = verbose
	ifile = infile
	ofile = outfile
	odir = outdir
	oform1 = outform1
	oform2 = outform2
	oform = outform
	makeout = no
	if (ofile == "") {
	    makeout = yes
	    }
	delint = no
	sfile = "skytemp.fits"
	jfile = intfile
	if (jfile == "") {
	    jfile = "eigtemp1.fits"
	    delint = yes
	    }
	delf = delfile

	svdres.specband = specband
	svdres.specnum = specnum
	svdprep.specband = specband
	svdprep.specnum = specnum
	svdres.debug = debug
	svdres.res_plot = plot
	svdres.plot_int = cursor
	svdres.badlines = badlines

	set clobber = yes

#  Process data files from RFN or file of RFN's
	tchar = substr (ifile,1,1)
	if (tchar != '@') {
	    filelist = mktemp ("eiglist")
	    print (ifile,>filelist)
	    listfile = filelist
	    }
	else
	    listfile = substr(ifile,2,64)

	if (vb)
	    print ("ESKY: files from ",listfile)
 
#  Loop through list of reduced files
	list = listfile
	i = 0
	while (fscan (list, line) != EOF) {
	    i = i + 1
	    if (indir != "") {
		localfile = indir // "/" // line
		}
	    else {
		localfile = line
		}

	# If the reduced file is found, process a local copy of it
	    if (localfile != "") {
		cfile = "svdpass1in.fits"
		tfile = "svdpass2in.fits"

	# Set up first pass continuum-removed file
		imgets (localfile, "OBJECT")
		object = imgets.value
		svdprep.compname = object // " - cont"
		svdprep.fixbad = fixbad1
		svdprep.normout = renorm1
		contprep.order = order1
		svdprep.cont_remove = "subtract"
		svdprep (localfile, cfile)

	# Fit first pass eigenvectors
		svdres.outname = object // " SVD pass 1 res"
		svdres.eigenfiles = vecdir // vecfile1
		svdres.eigenum = vectors1
		svdres.fixbad = fixbad1
		svdres.renormalize = 0.0
		svdres.cont_remove = "no"
		svdres.output_format = oform1
		svdres (cfile, jfile)

	# Set up second pass continuum-removed file
		svdprep.compname = object // " SVD pass 1 res - cont"
		svdprep.fixbad = fixbad2
		svdprep.normout = renorm2
		contprep.order = order2
		svdprep.cont_remove = "subtract"
		svdprep (jfile, tfile)

	# Fit second pass eigenvectors
		svdres.eigenfiles = vecdir // vecfile2
		svdres.eigenum = vectors2
		svdres.fixbad = fixbad2
		svdres.renormalize = 0.0
		svdres.cont_remove = "no"
		svdres.outname = object // " SVD pass 2 res"
		contres.order = order2
		svdres.output_format = oform2
		if (makeout) {
		    fileroot (localfile)
		    ofile = odir // fileroot.root // "s" // fileroot.ext
		    }
		svdres (tfile, ofile)
		if (delint) {
		    imdelete (tfile)
		    imdelete (cfile)
		    imdelete (jfile)
		    }
		if (oform == "multispec") {
		    imarith (ifile, "-", ofile, sfile)
		    imstack ofile,ifile,sfile
		    ffile = outdir // fileroot.root // "s.ms.fits" 
		    rename (imstack.fits, ffile)
		    }
		}
	    }

	if (tchar != '@')
	    delete (filelist)
end

# May 20 1999	New task
# Jul 15 1999	Add continuum averaging parameters
# Jul 30 1999	Add per-pass renormalization and fixbad parameters
# Sep  8 1999	Set badlines
# Sep 17 1999	Change eigenres task to svdres
# Oct  8 1999	Fix bug which failed to set titles
#
# Jan 27 2000	Drop continuum averaging parameters; reset defaults
# Feb 22 2000	Add directory for eigenvectors
# Feb 22 2000	Create output file name from input if no output file given
# Feb 22 2000	Add output directory
# Feb 23 2000	Debug
