# File svdfit/eskyv.cl
# January 28, 2000
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# ESKY -- Remove sky from a spectrum by fitting eigenvectors for
#         a file or list of files and cross-correlate redshifts

procedure esky (infile, outfile)

string	infile=""	{prompt="Input spectrum file"}
string	specnum="1"	{prompt="Range of spectra in multispec or 2D file"}
int	specband=1	{prompt="Band if multispec file"}
string	indir=""	{prompt="Input directory"}
string	outform="multispec" {prompt="Output format (multispec|oned|twod)"}
string	intfile="eigtemp1.fits"	{prompt="Intermediate spectrum file"}
file	vecfile1="evec1.fits"	{prompt="First pass eigenvector file"}
string	vectors1="1-2"	{prompt="First pass eigenvectors"}
string	fixbad1="zero"	{prompt="Fix bad lines in first pass (zero fill no)"}
real	renorm1=1.0	{prompt="Renormalize (divide by mean) in first pass"}
int	order1=40	{prompt="First pass continuum order"}
bool	delfile=yes	{prompt="Delete intermediate files (y or n)"}
string	outfile="eigtemp2.fits"	{prompt="Output spectrum file"}
file	vecfile2="evec2.fits"	{prompt="Second pass eigenvector file"}
string	fixbad2="zero"	{prompt="Fix bad lines in second pass (zero fill no)"}
real	renorm2=0.0	{prompt="Renormalize (divide by mean) in second pass"}
string	vectors2="1-10"	{prompt="Second pass eigenvectors"}
int	order2=40	{prompt="Second pass continuum order"}
file	badlines="badskynao.dat" {prompt="File with lines to eliminate"}
bool	plot=yes	{prompt="Plot results on display (y or n)"}
bool	cursor=no	{prompt="Wait after plot with cursor (y or n)"}
bool	verbose=no	{prompt="Print what's happening (y or n)"}
bool	debug=no	{prompt="Debugging mode (y or n)"}

begin

bool	vb, delf
int	i
file	filelist
file	localfile, ifile, jfile, ofile
file	listfile, eigtemp
file	cfile, tfile
string	tchar, object

	ifile = infile
	ofile = outfile
	vb = verbose
	jfile = intfile
	delf = delfile

	svdres.specband = specband
	svdres.specnum = specnum
	svdprep.specband = specband
	svdprep.specnum = specnum
	xcsao.specnum = specnum
	svdres.debug = debug
	svdres.res_plot = plot
	svdres.plot_int = cursor
	svdres.badlines = badlines
	xcsao.badlines = badlines

	set clobber = yes

#  Process data files from RFN or file of RFN's
	tchar = substr (ifile,1,1)
	if (tchar != '@') {
	    filelist = mktemp ("eiglist")
	    print (ifile,>filelist)
	    listfile = filelist
	    }
	else
	    listfile = substr(ifile,2,64)

	if (vb)
	    print ("ESKY: files from ",listfile)
 
#  Loop through list of reduced files
	list = listfile
	i = 0
	while (fscan (list, line) != EOF) {
	    i = i + 1
	    if (indir != "") {
		localfile = indir // "/" // line
		}
	    else {
		localfile = line
		}

	# If the reduced file is found, process a local copy of it
	    if (localfile != "") {
		cfile = "svdpass1in.fits"
		tfile = "svdpass2in.fits"

	# Set up first pass continuum-removed file
		imgets (localfile, "OBJECT")
		object = imgets.value
		svdprep.compname = object // " - cont"
		svdprep.fixbad = fixbad1
		svdprep.normout = renorm1
		contprep.order = order1
		svdprep.cont_remove = "subtract"
		svdprep (localfile, cfile)

	# Fit first pass eigenvectors
		svdres.outname = object // " SVD pass 1 res"
		svdres.eigenfiles = vecfile1
		svdres.eigenum = vectors1
		svdres.fixbad = fixbad1
		svdres.renormalize = 0.0
		svdres.cont_remove = "no"
		svdres (cfile, jfile)

	# Set up second pass continuum-removed file
		svdprep.compname = object // " SVD pass 1 res - cont"
		svdprep.fixbad = fixbad2
		svdprep.normout = renorm2
		contprep.order = order2
		svdprep.cont_remove = "subtract"
		svdprep (jfile, tfile)

	# Fit second pass eigenvectors
		svdres.eigenfiles = vecfile2
		svdres.eigenum = vectors2
		svdres.fixbad = fixbad2
		svdres.renormalize = 0.0
		svdres.cont_remove = "no"
		svdres.outname = object // " SVD pass 2 res"
		contres.order = order2
		svdres (tfile, ofile)

	# Cross-correlate
		xcsao (ofile)
		}
	    }

	if (tchar != '@')
	    delete (filelist)
end

# Jan 28 2000	New task, based on esky1.cl
