# File sfdfit/svdprep/svdcont.x
# May 8, 2008
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics
# After IRAF RV.CONTINUUM subroutine

include <pkg/gtools.h>
include	<error.h>
include "contin.h"
include	"rvsao.h"
include	"emv.h"


# SVDCONT - Do the continuum normalization, with optional interaction

procedure svdcont (n, fdata, wltemp, name, nsm, fit, ymin, ymax)

int	n		# Number of pixels to fit
real	fdata[ARB]	# Spectrum to be fit
real	wltemp[ARB]	# Wavelengths for spectrum
char	name[ARB]	# Name of spectrum
int	nsm		# Number of times to smooth spectrum
real	fit[ARB]	# Fit array (Returned)
real	ymin, ymax	# minimum and maximum for plots

int	i,newrej,nrej,ns
char	title_line[SZ_LINE]
char	wlab[SZ_LINE]
real	cveval()
pointer	spec0
pointer	ic			# ICFIT ptr
pointer	gt			# GTOOLS pointer
pointer	gp			# graphics pointer
pointer	cv			# pointer for curve fitting
pointer	sp
pointer gopen(), gt_init()
errchk  gopen, gt_init

include "contin.com"
include	"rvsao.com"
include	"emv.com"

begin
	call strcpy ("Wavelength in Angstroms", wlab, SZ_LINE)

# Set the ICFIT pointer structure.
	call ic_open (ic)
	call ic_pstr (ic, "sample", sample)
	call ic_pstr (ic, "function", confunc)
	call ic_puti (ic, "naverage", naverage)
	call ic_puti (ic, "order", order)
	call ic_puti (ic, "niterate", niterate)
	call ic_putr (ic, "low", lowrej)
	call ic_putr (ic, "high", hirej)
	call ic_putr (ic, "grow", grow)
	call ic_pstr (ic, "ylabel", "")

#  Initlialize X and WTS array
	do i = 1, n
	    Memr[contbuf+i-1] = real (i)
	call amovkr (1., Memr[contwts], n)
	call amovki (NO, Memi[contrej], n)
	if (badproc == ZEROBAD)
	    call zerolist (n, Memi[contwts], wltemp, debug)

#  Update icfit struct
	call ic_putr (ic, "xmin", 1.)
	call ic_putr (ic, "xmax", real(n))

#  If the interactive flag is set then use icg_fit to set the fitting
#  parameters.  Only done if task is run interactively as well.

	if (interact) {
	    iferr {
		gp = gopen ("stdgraph", NEW_FILE, STDGRAPH)
		gt = gt_init ()
		call gt_sets (gt, GTTYPE, "line")
		} then
		    call error (0, "Error opening `stdgraph'.")

	    call icg_fit (ic,gp,"cursor",gt,cv,Memr[contbuf],fdata,
			  Memr[contwts],n)
	
	# Now recover any parameters that were changed
	    call recover_icfit_pars (ic)
	    call gt_free (gt)
	    call gclose (gp)
	    }

#  Do the fit non-interactively using ic_fit.
	else {
	    call ic_fit (ic,cv,Memr[contbuf],fdata,Memr[contwts],
			 n,YES,YES,YES,YES)
	    }

#  Find deviant points
	if (schop && (abrej > 0. || emrej > 0.)) {

#  Save original spectrum if plotting and chopping emission lines
	    call smark (sp)
	    if (pltcon) {
		call salloc (spec0,n,TY_REAL)
		do i = 1, n {
		    Memr[spec0+i-1] = fdata[i]
		    }
		}
	    nrej = 0
	    newrej = 0
	    call ic_deviantr (cv, Memr[contbuf],fdata,Memr[contwts],
			      Memi[contrej],n,abrej,emrej,grow,NO,
			      nrej,newrej)

#  Replace points out of bounds with continuum
	    do i = 1, n {
		if (Memi[contrej+i-1] == YES)
		    fdata[i] = cveval (cv, Memr[contbuf+i-1])
		}

# If plot enabled, plot the lines that were removed
	    if (pltcon && newrej > 0) {
		do i = 1, n {
		    Memr[spec0+i-1] = Memr[spec0+i-1] - fdata[i]
		    }
		call strcpy (name, title_line, SZ_LINE)
		if (emrej > abrej)
		    call strcat (" absorption lines", title_line, SZ_LINE)
		else
		    call strcat (" emission lines", title_line, SZ_LINE)
		ns = nsm
		call plotsum (n,Memr[spec0],title_line,wltemp,wlab,ns,ymin,ymax)
		}
	    call sfree (sp)
	    }

#  Now remove the continuum fit.
	if (debug) {
	    call printf ("SVDCONT: continuum fit about to be removed\n")
	    call flush (STDOUT)
	    }
	call cvvector (cv, Memr[contbuf], Memr[contvec], n)
	if (conproc == DIVCONT || conproc == ZEROCONT) {
	    do i = 1, n {
		if (Memr[contvec+i-1] <= 0.0) {
		    if (i > 1)
			Memr[contvec+i-1] = Memr[contvec+i-2]
		    else
			Memr[contvec+i-1] = 1.0
		    }
		}
	    call adivr (fdata, Memr[contvec], fdata, n)
	    if (conproc == ZEROCONT)
		call asubkr (fdata, 1.0, fdata, n)
	    }
	else if (conproc == SUBCONT)
	    do i = 1, n {
		if (fdata[i] == 0.0)
		    Memr[contvec+i-1] = 0.0
		}
	    call asubr (fdata, Memr[contvec], fdata, n)
	if (debug) {
	    call printf ("SVDCONT: continuum fit removed\n")
	    call flush (STDOUT)
	    }

# If plot enabled, show the object spectrum with continuum removed
	if (pltcon) {
	    call strcpy (name, title_line, SZ_LINE)
	    call strcat (" continuum removed", title_line, SZ_LINE)
	    ns = nsm
	    call plotsum (n,fdata,title_line,wltemp,wlab,ns,ymin,ymax)
	    }

	call ic_closer (ic)
	call cvfree (cv)
	return
end


# RECOVER_ICFIT_PARS - Since the ICFIT parameters may have been changed in
# an interactive operation, we need to get the new values from the ICFIT
# structure.

procedure recover_icfit_pars (ic)

pointer	ic		# ICFIT pointer

int	strdic(), ic_geti()
real	ic_getr()
include "contin.com"

begin
	naverage = ic_geti (ic, "naverage")
	order = ic_geti (ic, "order")
	niterate = ic_geti (ic, "niterate")
	lowrej = ic_getr (ic, "low")
	hirej = ic_getr (ic, "high")
	grow = ic_getr (ic, "grow")

	call ic_gstr (ic, "sample", sample, SZ_LINE)
	call ic_gstr (ic, "function", confunc, SZ_LINE)
	function = strdic (confunc, confunc, SZ_LINE, CN_INTERP_MODE)
	if (function == 0)
	    call error (0, "Unknown fitting function type")
	return
end
# Nov 18 1991	Use malloc instead of salloc
# Nov 20 1991	Free buffers before allocating them
# Dec 16 1991	Plot spectrum and emission lines removed
#		Fix bug by filling in rejected points
#		Chop emission lines only if chop is set
# Dec 17 1991	Don't plot emission lines if none have been removed

# Oct 22 1992	Always free spec0 before returning

# Apr 13 1994	Cleanup code after ftnchek
# Apr 26 1994	Clean up code
# May  3 1994	Add smoothed vector argument to PLOTSPEC call
# Jun 15 1994	Add smoothing argument to PLOTSPEC call
# Aug  3 1994	Use separate template and spectrum emission line chopping flags
# Aug  3 1994	Change common and header from fquot to rvsao

# Mar 13 1995	Allow line chopping in absorption as well as emission
# Mar 15 1995	Add argument for name of spectrum
# May 15 1995	Add template-driven option to divide, not subtract, continuum
# Oct 16 1995	Try salloc instead of malloc

# Dec 13 1996	Remove zeroes and negative numbers from continuum if dividing
# Dec 16 1996	Subtract one after dividing continuum

# May  6 1997	Add smoothing parameter as argument
# Dec 22 1997	Add units to wavelength graph label

# Aug 16 1999	Comment out subtraction of 1 when dividing out continuum
# Aug 18 1999	Add ZEROCONT for continuum divided - 1
# Aug 18 1999	Use int conproc instead of bool divcon

# Jan 14 2000	Make rejection flags singular
# Jan 18 2000	Pass buffers instead of allocating them here
# Jan 19 2000	Zero-weight bad pixels from list before fitting
# Jan 19 2000	If subtracting, do not change data pixels with value of 0.0
# May 10 2000	Use zerolist() instead of rejlist()

# May 8 2008	Add ymin and ymax arguments and call plotsum(), not plotspec()
