# File svdfit/Eigenres/eigencon.x
# February 5, 2009
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics
# After IRAF RV.CONTINUUM subroutine

include <pkg/gtools.h>
include	<error.h>
include	"rvsao.h"
include	"contin.h"
include	"emv.h"


# EIGENCON - Fit and subtract a continuum

procedure eigencon (n, fdata, wldata, name, nsm, ispec, nspec, isvec, lndiff, cnfit, dmean)

int	n			# Number of pixels to fit
real	fdata[ARB]		# Spectrum to be fit
real	wldata[ARB]		# Wavelengths for spectrum
char	name[ARB]		# Name of spectrum
int	nsm			# Number of times to smooth spectrum
int	ispec			# Sequence number of spectrum
int	nspec			# Number of spectra in file
bool	isvec			# True if processing eigenvector
real	lndiff[ARB]		# Removed lines (returned)
real	cnfit[ARB]		# Removed continuum (returned)
double	dmean			# mean flux in counts per pixel (returned)

int	i,ns, i1, n1
char	title_line[SZ_LINE]
char	wlab[SZ_LINE]
char	tstring[16]
pointer	ic			# ICFIT ptr
pointer	gt			# GTOOLS pointer
pointer	gp			# graphics pointer
pointer	cv			# pointer for curve fitting
pointer gopen(), gt_init()
errchk  gopen, gt_init

include "contin.com"
include	"rvsao.com"
include	"emv.com"

begin
	call strcpy ("Wavelength in Angstroms", wlab, SZ_LINE)

	# Fill in or zero bad lines before removing continuum
	if (badproc != NOBAD) {
	    do i = 1, n {
		lndiff[i] = fdata[i]
		}
	    if (badproc == FILLBAD || badproc == NZEROBAD) {
		call filllist (n, fdata, wldata, debug)
		call strcpy ("filled", tstring, 16)
		}
	    else if (badproc == ZEROBAD) {
		call zerolist (n, fdata, wldata, debug)
		call strcpy ("zeroed", tstring, 16)
		}
	    if (pltspec) {
		if (nspec > 0) {
		    call sprintf (title_line, SZ_LINE, "%d: %s bad lines %s")
			call pargi (ispec)
			call pargstr (name)
			call pargstr (tstring)
		    }
		else {
		    call sprintf (title_line, SZ_LINE, "%s bad lines %s")
			call pargstr (name)
			call pargstr (tstring)
		    }
		ns = nsm
		call plotspec (n, fdata, title_line, wldata, wlab, ns)
		}
	    do i = 1, n {
		lndiff[i] = lndiff[i] - fdata[i]
		}
	    if (debug) {
		if (isvec)
		    call printf ("EIGENCON: Bad lines removed from eigenvector\n")
		else
		    call printf ("EIGENCON: Bad lines removed from spectrum\n")
		}
	    }
	else {
	    do i = 1, n {
		lndiff[i] = 0.0
		}
	    if (debug) {
		if (isvec)
		    call printf ("EIGENCON: No bad lines removed from eigenvector\n")
		else
		    call printf ("EIGENCON: No bad lines removed from spectrum\n")
		}
	    }
	if (isvec) {
	    if (convec == NOCONT) {
		if (debug)
		    call printf ("EIGENCON: No continuum removed from eigenvector\n")
		return
		}
	    }
	else if (conproc == NOCONT) {
	    if (debug)
		call printf ("EIGENCON: No continuum removed from spectrum\n")
	    return
	    }

# Set the ICFIT pointer structure.
	call ic_open (ic)
	call ic_pstr (ic, "sample", sample)
	call ic_pstr (ic, "function", confunc)
	call ic_puti (ic, "naverage", naverage)
	call ic_puti (ic, "order", order)
	call ic_puti (ic, "niterate", niterate)
	call ic_putr (ic, "low", lowrej)
	call ic_putr (ic, "high", hirej)
	call ic_putr (ic, "grow", grow)
	call ic_pstr (ic, "ylabel", "")

#  Initlialize X and WTS array
	do i = 1, n
	    Memr[contbuf+i-1] = real (i)
	call amovkr (1., Memr[contwts], n)
	call amovki (NO, Memr[contrej], n)
	if (badproc == ZEROBAD)
	    call zerolist (n, Memi[contwts], wldata, debug)

#  Avoid fitting zeroes at ends of spectrum
	i1 = 1
        do i = 1, n {
            if (fdata[i] == 0.) {
                Memr[contwts+i-1] = 0.
		i1 = i1 + 1
                }
            else
                break
            }
	n1 = n - i1 + 1
        do i = n, 1, -1 {
            if (fdata[i] == 0.) {
                Memr[contwts+i-1] = 0.
		n1 = n1 - 1
                }
            else
                break
            }

# Compute mean flux in counts
	dmean = 0.0
	do i = i1, n1 {
	    dmean = dmean + (fdata[i] * fdata[i])
	    }
	dmean = dsqrt (dmean) / double (n1 - i1 + 1)

#  Update icfit struct
	call ic_putr (ic, "xmin", 1.)
	call ic_putr (ic, "xmax", real(n))

#  If the interactive flag is set then use icg_fit to set the fitting
#  parameters.  Only done if task is run interactively as well.

	if (interact) {
	    iferr {
		gp = gopen ("stdgraph", NEW_FILE, STDGRAPH)
		gt = gt_init ()
		call gt_sets (gt, GTTYPE, "line")
		} then
		    call error (0, "Error opening `stdgraph'.")

	    call icg_fit (ic,gp,"cursor",gt,cv,Memr[contbuf],fdata,Memr[contwts],n)
	
	# Now recover any parameters that were changed
	    call recover_icfit_pars (ic,1)
	    call gt_free (gt)
	    call gclose (gp)
	    }

#  Do the fit non-interactively using ic_fit.
	else {
	    call ic_fit (ic,cv,Memr[contbuf],fdata,Memr[contwts],n,YES,YES,YES,YES)
	    }
	if (debug) {
	    call printf ("EIGENCON: %d-pixel continuum fit\n")
		call pargi (n)
	    call flush (STDOUT)
	    }

#  Compute the continuum at each spectrum pixel
	call cvvector (cv, Memr[contbuf], cnfit, n)

	if (debug) {
	    call printf ("EIGENCON: Continuum has been computed\n")
	    call flush (STDOUT)
	    }

#  Subtract the continuum fit (but not from zeroes at ends of spectrum)
	call asubr (fdata[i1], cnfit, fdata[i1], n1)

	if (debug) {
	    call printf ("EIGENCON: Continuum has been subtracted\n")
	    call flush (STDOUT)
	    }

# If plot enabled, show the continuum and object spectrum with continuum removed
	if (pltcon) {
	    call strcpy (name, title_line, SZ_LINE)
	    call strcat (" continuum", title_line, SZ_LINE)
	    ns = nsm
	    call plotspec (n,cnfit,title_line,wldata, wlab, ns)
	    call strcpy (name, title_line, SZ_LINE)
	    call strcat (" continuum removed", title_line, SZ_LINE)
	    ns = nsm
	    call plotspec (n,fdata,title_line,wldata, wlab, ns)
	    }

	call ic_closer (ic)
	call cvfree (cv)

	# Zero bad lines after removing continuum
        if (badproc == NZEROBAD) {
	    call zerolist (n, fdata, wldata, debug)
	    if (pltspec) {
		if (nspec > 0) {
		    call sprintf (title_line, SZ_LINE, "%d: %s bad lines zeroed")
			call pargi (ispec)
			call pargstr (name)
		    }
		else {
		    call sprintf (title_line, SZ_LINE, "%s bad lines zeroed")
			call pargstr (name)
		    }
		ns = nsm
		call plotspec (n, fdata, title_line, wldata, wlab, ns)
		}
	    }
	return
end

# May  3 1999	New subroutine based on rvsao.icsubcon.x

# Jan 18 2000	Pass buffers in contin.com
# Jan 19 2000	Zero-weight bad pixels from list before fitting
# Jan 19 2000	Set continuum fit to zero at zeroed data points
# Apr 26 2000	Include bad line removal
# May 10 2000	Use zerolist() instead of rejlist()

# Dec 29 2005	Return fit continuum and check for eigenvector processing

# May  2 2006	Plot continuum as well as spectrum - continuum
# Nov 16 2006	Return mean flux

# May 29 2007	Save chopped emission lines

# Feb  5 2009	Add debugging when not removing lines or continuum
