# File rvsao/Util/getvec.x
# November 20, 2006
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics
# After Gerard Kriss, Johns Hopkins University

# Copyright(c) 1994-2006 Smithsonian Astrophysical Observatory
# You may do anything you like with this file except remove this copyright.
# The Smithsonian Astrophysical Observatory makes no representations about
# the suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.
 
# GETVEC opens one eigenvector spectrum, extracts wavelength and velocity
# information from the header, and returns the spectrum.
# Data is shared in common in "rvsao.com"
 
include <imhdr.h>
include	"rvsao.h"
include	<smw.h>
define  LEN_USER_AREA   2880
 
procedure getvec (vecfile,mspec,mband,vecdir,vecim,vecsh,itemp)
 
char	vecfile[ARB]	# Template file name
int	mspec		# Spectrum aperture to read from multispec or 2-D file
int	mband		# Spectrum band to read from multispec file
char	vecdir[ARB]	# Template directory
pointer	vecim		# Template image header structure (returned)
pointer	vecsh		# Template spectrum header structure (returned)
int	itemp		# Template number for saved information

char	vecpath[SZ_PATHNAME]
int	world		# 1=wavelength or 2=pixel
int	mode, npix, mext
char	vecnm[SZ_PATHNAME]
pointer	vecspec		# Template spectrum

include	"rvsao.com"

begin
	mode = READ_ONLY
	world = 1
	mext = 0

	call strcpy (vecdir,vecpath,SZ_PATHNAME)
	call strcat (vecfile,vecpath,SZ_PATHNAME)
	if (debug) {
	    call printf ("GETVEC: %d %s %d %d\n")
		call pargi (itemp)
		call pargstr (vecpath)
		call pargi (mspec)
		call pargi (mband)
	    }

# Get template spectrum and image header
	call getimage (vecpath, mext, mspec, mband, vecspec, vecim, vecsh, npix,
		       vecnm, mode, world, debug)

	if (vecim == ERR) {
	    call printf ("GETVEC: Cannot read %s\n")
		call pargstr (tempname[1,itemp])
	    return
	    }
	call strcpy (vecnm, tempname[1,itemp], SZ_PATHNAME)
	if (debug) {
	    call printf ("GETVEC: %s  %d pixels read\n")
		call pargstr (tempname[1,itemp])
		call pargi (npix)
#	    call printf ("GETVEC: %f %f %f\n")
#		call pargr (Memr[vecspec])
#		call pargr (Memr[vecspec+1])
#		call pargr (Memr[vecspec+2])
	    }

	temppix[itemp] = npix

	return
end
# Apr 28 1999	New subroutine based on rvsao/Util/gettemp.x

# Nov 20 2006	Modify to use new getimage()
