# File svdfit/Svdres/getvecs.x
# May 17, 2000
# By Doug Mink, after Bill Press and Mike Kurtz

# Load all eigenvectors for one pass

include <imhdr.h>
include <imio.h>
include <smw.h>
include "rvsao.h"

procedure getvecs (vecfiles, vecdir, vecband, vecnums0, minwav, maxwav, dwav,
		   spwl1, spwl2, rebvec, nbin, rebwl, ipass, ivec)

char	vecfiles[SZ_PATHNAME]	# List of eigenvector spectra
char	vecdir[SZ_PATHNAME]	# Directory for eigenvector spectra
int	vecband			# Band of multispec spectra to read
char	vecnums0[SZ_LINE]	# List of multispec spectra to read
double	minwav, maxwav		# Wavelength limits (parameter in, returned)
double	dwav			# Wavelength per pixel
double	spwl1, spwl2		# Spectrum wavelength limits
pointer	rebvec[ARB]		# Pointers to eigenvector pixels
int	nbin			# Number of bins in rebinned spectra
real	rebwl[ARB]		# Wavelength vector for rebinned spectra
int	ipass			# SVD pass (1 or 2)
int	ivec			# Number of eigenvectors read (returned)

char	vecfile[SZ_PATHNAME]	# Template spectrum file name
char	vecpath[SZ_PATHNAME]	# Template spectrum path name
char	vecnums[SZ_LINE]	# List of multispec spectra to read
pointer	veclist			# List of eigenvector files
char	title[SZ_LINE]
char	xlab[SZ_LINE]		# Title for wavelength plots of spectrum
int	vec_range[3,MAX_RANGES]
int	ip, jp, iv, nnums, iv1
int	tspec, lfile
double	dindef
double	minwav0, maxwav0
double	minwav1, maxwav1
double	tmwl1, tmwl2, tmdwl
double	tmshft
int	j
int	tpix1, tpix2, tempwcs, twlwcs, nptemp
pointer	vecim, vecsh

int	stridxs(), strlen(), stridx()
int	decode_ranges(),get_next_number()
int	imtgetim(), imaccess()
pointer	imtopen()
double	wcs_p2w

char	lbracket[8]		# left bracket, parenthesis, brace
char	rbracket[8]		# right bracket, parenthesis, brace
char	cdot			# Period
errchk	malloc

include	"rvsao.com"
include "svdvec.com"

define  newvec_	40
define  newtap_	42
define  nextvec_ 44

begin
	call strcpy ("[({",lbracket,8)
	call strcpy ("])}",rbracket,8)
	cdot = '.'
	dindef = INDEFD

	minwav0 = minwav
	maxwav0 = maxwav

	ivec = 0
	if (debug) {
	    call printf ("GETVECS: Pass %d nvec= %d, ncvec = %d\n")
		call pargi (ipass)
		call pargi (nvec[ipass])
		call pargi (ncvec[ipass])
	    }

	if (ncvec[ipass] == 0) {
	    ntemp = 0
	    veclist = imtopen (vecfiles)
	    }
	if (ipass == 2)
	    iv1 = 1 + ncvec[1]
	else
	    iv1 = 1

# Initialize new eigenvector file
newvec_
	if (ncvec[ipass] > 0) {
	    if (nvec[ipass] > 0 && ivec >= nvec[ipass])
		return
	    go to nextvec_
	    }
	tspec = -1

# Get next eigenvector spectrum file name from the list
	if (imtgetim (veclist, vecfile, SZ_PATHNAME) == EOF) {
	    nvec[ipass] = ivec
	    if (iv < MAXTEMPS)
		ncvec[ipass] = ivec
	    else if (ipass == 2)
		ncvec[ipass] = MAXTEMPS - ncvec[1]
	    else
		ncvec[ipass] = MAXTEMPS
	    return
	    }

# Check for specified apertures in multispec eigenvector file
	ip = stridxs (lbracket,vecfile)
	nnums = 0
	if (ip > 0) {
	    lfile = strlen (vecfile)
	    vecfile[ip] = EOS
	    ip = ip + 1
	    jp = 1
	    while (stridx (vecfile[ip],rbracket) == 0 && ip <= lfile) {
		vecnums[jp] = vecfile[ip]
		vecfile[ip] = EOS
		ip = ip + 1
		jp = jp + 1
		}
	    vecnums[jp] = EOS
	    if (decode_ranges (vecnums,vec_range,MAX_RANGES,nnums) == ERR)
		call error (1, "GETVECS: Illegal eigenvector multispec list")
	    }
	else {
	    call strcpy (vecnums0,vecnums,SZ_LINE)
	    if (decode_ranges (vecnums,vec_range,MAX_RANGES,nnums) == ERR)
		call error (1, "GETVECS: Illegal eigenvector multispec list")
	    }
	if (debug) {
	    call printf ("%d->%d - %s\n")
		call pargi (ivec)
		call pargi (iv)
		call pargstr (vecfile)
	    }

# Check for readability of eigenvector spectrum
	if (stridx ("/",vecfile) == 1 || stridx ("$",vecfile) > 0 ||
	    stridx (cdot,vecfile) == 1)
	    call strcpy (vecfile,vecpath,SZ_PATHNAME)
	else {
	    call strcpy (vecdir,vecpath,SZ_PATHNAME)
	    call strcat (vecfile,vecpath,SZ_PATHNAME)
	    }
	if (imaccess (vecpath, READ_ONLY) == NO) {
	    call eprintf ("GETVECS: cannot read eigenvector file %s \n")
		call pargstr (vecpath)
	    go to newvec_
	    }

# Get next multispec aperture number from list
newtap_
	if (get_next_number (vec_range, tspec) == EOF)
	    go to newvec_

# Start new eigenvector here if it has been cached
nextvec_
	ivec = ivec + 1

# Get position of vector in cache
	if (ipass == 2)
	    iv = ivec + ncvec[1]
	else
	    iv = ivec

# Use saved eigenvector spectrum
	if (ncvec[ipass] > 0 && ivec <= ncvec[ipass]) {
	    vecsh = tmpsh[iv]
	    vecim = tmpim[iv]
	    tspec = tmpspec[iv]
	    if (debug) {
		call printf ("GETVECS: Vector %d (%d) from cache vector %d\n")
		    call pargi (ivec)
		    call pargi (tspec)
		    call pargi (iv)
		call flush (STDOUT)
		}
	    call sprintf (title, SZ_LINE, "%d: %s eigenvector from cache\n")
		call pargi (ivec)
		call pargstr (tempid[1,iv])
	    }

# Load eigenvector spectrum
	else {
	    call getvec (vecfile,tspec,vecband,vecdir,vecim,vecsh,ivec)
	    if (vecim == ERR)
		go to newtap_

	    if (tspec > 0) {
		call sprintf (tempid[1,iv], SZ_PATHNAME,"%s[%d]")
		    call pargstr (vecfile)
		    call pargi (tspec)
		}
	    else {
		call sprintf (tempid[1,iv], SZ_PATHNAME, "%s")
		    call pargstr (vecfile)
		}
	    if (iv < MAXTEMPS) {
		tmpsh[iv] = vecsh
		tmpim[iv] = vecim
		tmpspec[iv] = tspec
		}
	    call sprintf (title, SZ_LINE, "%d: %s eigenvector original\n")
		call pargi (ivec)
		call pargstr (tempid[1,iv])
	    if (debug) {
		call printf ("GETVECS: %s: vector %d (%d) to cache vector %d\n")
		    call pargstr (tempid[1,iv])
		    call pargi (ivec)
		    call pargi (tspec)
		    call pargi (iv)
		call flush (STDOUT)
		}
	    }
	if (plttemp) {
	    tpix1 = NP1(vecsh)
	    tpix2 = NP2(vecsh)
	    tempwcs = SY(vecsh) + tpix1 - 1
	    twlwcs = SX(vecsh) + tpix1 - 1
	    nptemp = tpix2 - tpix1 + 1
	    call strcpy (LABEL(vecsh), xlab, SZ_LINE)
	    if (strlen (xlab) <= 0)
		call strcpy ("Wavelength", xlab, SZ_LINE)
	    if (strlen (UNITS(vecsh)) > 0) {
		call strcat (" in ",xlab,SZ_LINE)
		call strcat (UNITS(vecsh),xlab,SZ_LINE)
		}
	    else
		call strcat (" in Angstroms",xlab,SZ_LINE)
	    call plotspec (nptemp,Memr[tempwcs],title,Memr[twlwcs],xlab,
			   nsmooth)
	    }

# Set pixel limits of eigenvector WCS and plot eigenvector, if requested
	tpix1 = NP1(vecsh)
	tpix2 = NP2(vecsh)
	tempwcs = SY(vecsh) + tpix1 - 1
	twlwcs = SX(vecsh) + tpix1 - 1
	nptemp = tpix2 - tpix1 + 1

# Use wavelength limit parameters if they are present
	if (minwav0 == dindef && maxwav0 == dindef)
	    overlap = TRUE
	else
	    overlap = FALSE
	if (debug) {
	    if (overlap)
		call printf ("GETVECS: Entire overlapping wavelength space\n")
	    else
		call printf ("GETVECS: Wavelength limited by parameters\n")
	    call flush (STDOUT)
	    }

	call wcs_set (vecsh)
	tmwl1 = wcs_p2w (double (tpix1))
	tmwl2 = wcs_p2w (double (tpix2))
	tmdwl = (tmwl2 - tmwl1) / double (nptemp - 1)
	if (debug) {
	    call printf ("GETVECS: Eigenvector %d from %.4f to %.4f by %.5f")
		call pargi (ivec)
		call pargd (tmwl1)
		call pargd (tmwl2)
		call pargd (tmdwl)
	    call printf (" = %d pixels\n")
		call pargi (nptemp)
	    call flush (STDOUT)
	    }

	# Check to make sure that eigenvector and spectrum overlap
	if (spwl1 > tmwl2) {
	    call printf ("GETVECS: No overlap, spectrum starts at %9.2f angstroms\n")
		call pargd (spwl1)
	    call printf ("GETVECS: No overlap, template ends at   %9.2f angstroms\n")
		call pargd (tmwl2)
	    go to newvec_
	    }
	if (spwl2 < tmwl1) {
	    call printf ("GETVECS: No overlap, spectrum ends at   %9.2f angstroms\n")
		call pargd (spwl2)
	    call printf ("GETVECS: No overlap, template starts at %9.2f angstroms\n")
		call pargd (tmwl1)
	    go to newvec_
	    }

	# Figure lambda and log-lambda limits and increment
	minwav1 = max (spwl1, tmwl1)
	if ((overlap) || (minwav0==dindef) || (minwav0<minwav1))
	    minwav = minwav1
	else
	    minwav = minwav0

	maxwav1 = min (spwl2, tmwl2)
	if ((overlap) || (maxwav0==dindef) || (maxwav0>maxwav1))
	    maxwav = maxwav1
	else
	    maxwav = maxwav0
	if (nbin == INDEFI)
	    nbin = nptemp
	dwav = (maxwav - minwav) / double (nbin - 1)

	# Set up wavelength vector for rebinned eigenvectors
	if (ivec == 1 || minwav != twl1[ivec-1] || maxwav != twl2[ivec-1]) {
	    do j = 1, nbin {
		rebwl[j] = minwav + (double (j - 1) * dwav)
		}
	    }

	if (debug) {
	    call printf ("GETVECS: Cached %d/%d vectors %.2f - %.2f\n")
		call pargi (ncvec[ipass])
		call pargi (ivec)
		call pargd (minwav)
		call pargd (maxwav)
	    call flush (STDOUT)
	    }

	# If not cached, initialize this eigenvector
	if (ncvec[ipass] < ivec || (ncvec[ipass] >= ivec &&
	    (minwav != twl1[iv1] || maxwav != twl2[iv1]))) {

	    #  Rebin eigenvector to overlap this spectrum
	    if (debug) {
		call printf("GETVECS: eigenvector %9.7f to %9.7f by %.7f")
		    call pargd (minwav)
		    call pargd (maxwav)
		    call pargd (dwav)
		call printf(" = %d pixels\n")
		    call pargi (nbin)
		}
	    twl1[iv] = minwav
	    twl2[iv] = maxwav
	    dwl[iv] = dwav
	    if (rebvec[ivec] == NULL) {
		if (debug) {
		    call printf ("GETVECS: Allocating %d pixels for vector %d\n")
			call pargi (nbin)
			call pargi (ivec)
		    call flush (STDOUT)
		    }
		call malloc (rebvec[ivec], nbin, TY_REAL)
		}
	    tmshft = 0.d0
	    if (tmwl1 == minwav && tmdwl == dwav && nptemp == nbin) {
		if (debug) {
		    call printf ("GETVECS: copying %d-pixel eigenvector\n")
			call pargi (nbin)
		    call flush (STDOUT)
		    }
		call amovr (Memr[tempwcs],Memr[rebvec[ivec]],nbin)
		}
	    else {
		if (debug) {
		    call printf ("GETVECS: rebinning %d-pixel eigenvector\n")
			call pargi (nbin)
		    call flush (STDOUT)
		    }
		call rebin (Memr[tempwcs], vecsh, tmshft,
			Memr[rebvec[ivec]], nbin, minwav, dwl[ivec])
		if (plttemp) {
		    call sprintf (title, SZ_LINE, "%d: %s eigenvector rebinned\n")
			call pargi (tmpspec[ivec])
			call pargstr (tempid[1,ivec])
		    call plotspec (nbin,Memr[rebvec[ivec]],title,rebwl,xlab,nsmooth)
		    }
		}
	    }
	if (debug) {
	    call printf ("GETVECS: Pass %d, vector %d = %d: %f\n")
		call pargi (ipass)
		call pargi (ivec)
		call pargi (iv)
		call pargr (Memr[rebvec[ivec]])
	    call flush (STDOUT)
	    }

	# Reset per-eigenvector pointers
	vecim = NULL
	vecsh = NULL
	nnums = nnums - 1
	if (nnums > 0)
	    go to newtap_
	go to newvec_
end

# May 17 2000	New subroutine, out of eigenfit
