/*** File t_svdspec.x
 *** By Mike Kurtz, modified by Doug Mink
 *** July 26, 1999
 */

include <imhdr.h>
include <smw.h>
define	LEN_USER_AREA	10000

procedure t_svdspec ()

/*  Take an array of spectra and decompose it into an orthogonal
 *  basis using Singular Value Decomposition */

int	ia, ib, m, n, nb, i, j, ier
double	a[2000,2000], b[2000,2000], s[2000], wk[4000]
pointer	im

char	specfile[256]
char	eigfile[256]

{
    ia = 2000;
    m = 2000;
    n = 2000;
    ib = 2000;
    nb = 2000;


/* Spectra to from which to compute eigenvectors */
        call clgstr ("spectra", specfile, SZ_PATHNAME)

/* Name of file to which to write eigenvectors */
        call clgstr ("eigvec", eigfile, SZ_PATHNAME)

    /*  Read stacked spectra with bad lines zeroed */
    read_stack (specfile, header, a, m, n);


    bij = b;
    for (i = 0; i < m; i++) {
	for (j = 0; j < m; j++) {
	    *(bij++) = 0.d0
	    }
	}
    bii = b;
    for (i = 0; i < m; i++) {
	*bii = 1.d0
	bii = bii + m + 1;
	}

    svdcomp (a,ia,m,n,b,ib,nb,s,wk,ier);

    /* Write eigenvectors as stacked spectra */
    write_stack (eigfile, header, a,ia,m,n,b,ib,nb,s,wk,ier);

}


/*  READ_STACK -- Read FITS file containing the array of spectra to decompose */

read_stack (imname, im, a, m, n)

char	imname[ARB]
pointer	im
double	a[2000,2000]
int	m, n, nx, ny, i, j, ndim

pointer	spectra
pointer	imgs2r()
pointer	immap()
errchk	immap()

{

    /* Open IRAF image */
    if (isiraf (name)) {
	iraffile = 1;
	if ((irafheader = irafrhead (name, &lhead)) != NULL) {
	    if ((header = iraf2fits (name, irafheader, lhead, &nbhead)) == NULL) {
		fprintf (stderr, "Cannot translate IRAF header %s/n",name);
		free (irafheader);
		return;
		}
	    if ((image = irafrimage (header)) == NULL) {
		hgets (header,"PIXFILE", 64, pixname);
		fprintf (stderr, "Cannot read IRAF pixel file %s\n", pixname);
		free (irafheader);
		free (header);
		return;
		}
	    }
	else {
	    fprintf (stderr, "Cannot read IRAF header file %s\n", name);
	    return;
	    }
	}

    /* Open FITS file */
    else {
	iraffile = 0;
	if ((header = fitsrhead (name, &lhead, &nbhead)) != NULL) {
	    if ((image = fitsrimage (name, nbhead, header)) == NULL) {
		fprintf (stderr, "Cannot read FITS image %s\n", name);
		free (header);
		return;
		}
	    }
	else {
	    fprintf (stderr, "Cannot read FITS file %s\n", name);
	    return;
	    }
	}

        ndim = IM_NDIM(im)
	nx = IM_LEN(im, 1)
	ny = IM_LEN(im, 2)
	spectra = imgs2r (im, 1, nx, 1, ny)

/*  Convert real*4 input spectra to double for computation */
	aji = a;
	spji = spectra;
	for (i = 0; i < 2000; i++) {
	    if (i <= ny) {
		for (j = 0; j < 2000; j++) {
		    aji++ = spji++;
		    }
		}
	    else {
		for (j = 0; j < 2000; j++) {
		    aji++ = 0.d0;
		    }
		}
	    }

	free (spectra);
	return;
}


/*  WRITE_STACK -- Write FITS file containing eigenvector fits */

write_stack (imname, im, a,ia,m,n,b,ib,nb,s)

char	imname[ARB]	/* Output eigenvector file */
pointer	im		/* Image descriptor */
double	a[ia,m], b[m,nb], s[n]
int	m, n, ia, ib, nb, nx, ny

int	i, j
pointer	imout		/* Output image descriptor */
pointer	imps2r()
pointer	immap()
errchk	immap()
pointer	vectors

begin

/*  Map the image */
	iferr (imout =  immap (imname, NEW_COPY, im)) {
	    if (imout != NULL) call imunmap (imout)
	    call eprintf ("WRITE_STACK: Cannot write image %s\n")
		call pargstr (imname)
	    return
	    }
	nx = 2000
	ny = 2000
	IM_LEN(imout, 1) = nx
	IM_LEN(imout, 2) = ny
	IM_PHYSLEN(imout, 1) = nx
	IM_PHYSLEN(imout, 2) = ny
	vectors = imps2r (imout, 1, nx, 1, ny)

/*  Convert double input coefficient vectors to real*4 for output */
	do i = 1, 2000 {
	    do j = 1, 2000 {
		Memr(vectors+j+(2000*(i-1))] = b[j,i]
		}
	    }

	call imunmap (imout)

	return
end
