#*** File t_svdspec.x
#*** By Mike Kurtz, modified by Doug Mink
#*** July 26, 1999

include <imhdr.h>
include <smw.h>
define	LEN_USER_AREA	10000

procedure t_svdspec ()

#  Take an array of spectra and decompose it into an orthogonal
#  basis using Singular Value Decomposition

int	ia, ib, m, n, nb, i, j, ier
double	a[2000,2000], b[2000,2000], s[2000], wk[4000]
pointer	im
bool debug

char	specfile[SZ_PATHNAME]
char	eigfile[SZ_PATHNAME]

bool	clgetb()

begin
	ia = 2000
	m = 2000
	n = 2000
	ib = 2000
	nb = 2000


# Spectra to from which to compute eigenvectors
        call clgstr ("spectra", specfile, SZ_PATHNAME)

# Name of file to which to write eigenvectors
        call clgstr ("eigvec", eigfile, SZ_PATHNAME)

	debug = clgetb ("debug")

#  Read stacked spectra with bad lines zeroed
	call read_stack (specfile, im, a, m, n)

	if (debug)
	    call printf ("SVDSPEC:  spectrum stack read\n")
	do i = 1, m {
	    do j = 1, m {
		b[i,j] = 0.d0
		}
	    }
	do i = 1, m {
	    b[i,i] = 1.d0
	    }
	if (debug)
	    call printf ("SVDSPEC:  eigenvectors initialized\n")
	call flush (STDOUT)

	call svdcomp (a,ia,m,n,b,ib,nb,s,wk,ier)

# Write eigenvectors as stacked spectra
	call write_stack (eigfile, im, a,ia,m,n,b,ib,nb,s,ier)

	call imunmap (im)

	stop
end


#  READ_STACK -- Read FITS file containing the array of spectra to decompose

procedure read_stack (imname, im, a, m, n)

char	imname[ARB]
pointer	im
double	a[2000,2000]
int	m, n, nx, ny, i, j, ndim

pointer	spectra
pointer	imgs2r()
pointer	immap()
errchk	immap()

begin

#  Map the image
	iferr (im = immap (imname, READ_ONLY, LEN_USER_AREA)) {
	    if (im != NULL) call imunmap (im)
	    call eprintf ("READ_STACK: Cannot read image %s\n")
		call pargstr (imname)
	    im = ERR
	    return
	    }
        ndim = IM_NDIM(im)
	nx = IM_LEN(im, 1)
	ny = IM_LEN(im, 2)
	spectra = imgs2r (im, 1, nx, 1, ny)

#  Convert real*4 input spectra to double for computation
	do i = 1, 2000 {
	    if (i <= ny) {
		do j = 1, 2000 {
		    a[j,i] = Memr[spectra+j+(2000*(i-1))]
		    }
		}
	    else {
		do j = 1, 2000 {
		    a[j,i] = 0.d0
		    }
		}
	    }

	return
end


#  WRITE_STACK -- Write FITS file containing eigenvector fits

procedure write_stack (imname, im, a,ia,m,n,b,ib,nb,s, ier)

char	imname[ARB]	# Output eigenvector file
pointer	im		# Image descriptor
double	a[ia,m], b[m,nb], s[n]
int	m, n, ia, ib, nb, nx, ny
int	ier

int	i, j
pointer	imout		# Output image descriptor
pointer	imps2r()
pointer	immap()
errchk	immap()
pointer	vectors

begin
	ier = 0

#  Map the image
	iferr (imout =  immap (imname, NEW_COPY, im)) {
	    if (imout != NULL) call imunmap (imout)
	    call eprintf ("WRITE_STACK: Cannot write image %s\n")
		call pargstr (imname)
	    ier = 1
	    return
	    }
	nx = 2000
	ny = 2000
	IM_LEN(imout, 1) = nx
	IM_LEN(imout, 2) = ny
	IM_PHYSLEN(imout, 1) = nx
	IM_PHYSLEN(imout, 2) = ny
	vectors = imps2r (imout, 1, nx, 1, ny)

#  Convert double input coefficient vectors to real*4 for output
	do i = 1, 2000 {
	    do j = 1, 2000 {
		Memr(vectors+j+(2000*(i-1))] = b[j,i]
		}
	    }

	call imunmap (imout)

	return
end
