c*** File fitslib/fitshersh.f
c*** August 26, 1996
c*** By A. Chave IGPP / UCSD Aug 1981
c*** Modified Feb 1982 by A. Chave, R.L. Parker, and L. Shure
c*** from Brian Kantor	UCSD Computer Graphics Lab
c			c/o B-028, La Jolla, CA 92093 (619)452-6865
c*** with additional modifications by Doug Mink, Center for Astrophysics

	Subroutine FITSHERSH (IMAGE,X,Y,HEIGHT,TEXT,THETA,NTEXT)

	Integer*4 IMAGE(*)
c			FITS image into which characters are being inserted
	Real*4 X,Y
c			Coordinates from the current origin to the
c			lower left corner of the 1st character to be plotted.
c			If either is less than 0, the saved next character
c			position is used.
	Real*4 HEIGHT
c			Character height in inches
	Character*(*) TEXT
c			Character string containing the text to be plotted
	Real*4 THETA
c			Positive CCW angle W.R.T. the x-axis
	Integer*4 NTEXT
c			Number of characters in TEXT to plot
c		 	If NTEXT<-1, the pen is down to (X,Y) and a single
c			special centered symbol is plotted. If NTEXT = -1,
c			the pen is up to (X,Y) and a single special centered
c			symbol is plotted. If NTEXT = 0, a single simplex
c			Roman character from TEXT, left-justified, is plotted.
c			If NTEXT > 0, NTEXT characters from TEXT are decoded
c			and NCHR characters are plotted where NCHR <= NTEXT
c			to remove backslashes, command codes, etc.

c  Features:
c    1) Four Hershey letter fonts--simplex,complex,italic, and duplex--
c	 are provided in upper and lower case Roman
c    2) Two Hershey letter fonts--simplex and complex--are provided in
c	 upper and lower case Greek
c    3) 47 special mathematical symbols, e.g. integral sign,del, are
c	 provided
c    4) Super- and sub-scripting is possible within a character string
c	 without separate calls to SYMBOL

c   Change of font is made by enclosing the name of the font in
c  backslashes, e.g \SIMPLEX\.  Three letters suffice to
c  specify the font.  Simplex is the default font on the initial call
c  to symbol.  A font remains in effect until explicitly changed.
c   Super- or sub-scripting is accomplished by enclosing the expression
c  to be super- or sub-scripted in curly brackets and preceding it by
c  sup or sub. The closing curly bracket terminates the
c  super- or sub-scripting and returns to normal character plotting.
c  Note that super- and sub-script letters are plotted with a
c  different character size.
c    Greek letters are drawn by enclosing the English name of the
c  letter in backslashes, e.g. \ALPHA\.  The case of the first
c  letter determines the case of the Greek letter.  The closing backslash
c  must be included.
c   Any symbol may be called by enclosing the symbol number + 1000 in
c  backslashes.  This is the only way to call some symbols,
c  especially special mathematical symbols.
c   The symbol numbers are:
c   1 - 26   upper case Roman simplex
c  27 - 52   lower case Roman simplex
c  53 - 72   simplex numbers and symbols
c  73 - 96   upper case Greek simplex
c  97 - 120  lower case Greek simplex
c  121 - 146 upper case Roman complex
c  147 - 172 lower case Roman complex
c  173 - 192 complex numbers and symbols
c  193 - 216 upper case Greek complex
c  217 - 240 lower case Greek complex
c  241 - 266 upper case Roman italic
c  267 - 292 lower case Roman italic
c  293 - 312 italic numbers and symbols
c  313 - 338 upper case Roman duplex
c  339 - 364 lower case Roman duplex
c  365 - 384 duplex numbers and symbols
c  385 - 432 special mathematical symbols
c  Additional features added Feb 1982

c  The pen may be moved back to the start point for the previous
c  character by \BS\.  This is useful, for example, in writing
c  integral signs with limits above and below them.
c
c  Symbol parameters taken from N.M.Wolcott, Fortran IV Enhanced
c  Character Graphics, NBS
c
c  A.Chave IGPP / UCSD Aug 1981, Modified Feb 1982 by A. Chave,
c					R.L. Parker, and L. Shure


c  Programmed in Fortran - 77

	Integer*4 I,L,IA,IS,IB,IPEN,IX,IY,ISAV,N,IC
	Real*4 OLDWID,SCALE,YOFF,SI,CO,XI,XO,YI,YO,XX,YY,RSCALE

	Common/AJUST/ NCHR,ICHR(350)
	Integer*4 NCHR,ICHR
c		ICHR(J) contains the symbol number of the Jth symbol or a
c		code to indicate space (1000),begin super-scripting (1001),
c		begin sub-scripting (1002), or end super/sub-scripting (1003),
c		or back-space (1004).

	Save XO,YO
	Integer*4 IXTRCT,NSTART,NBITS,IWORD

	Common/FOFFSET/ IOFF,JUST1,JUST2
	Integer*4 IOFF,JUST1,JUST2

	Integer*4 IUP,IDOWN
	Real*4 FACTOR,SUPSUB(2),RAISE(20)
	Real*4 RAD

c  Powers of two for IXTRCT
	Common/FBITS/ POW2
	Integer*4 POW2(0:31)

	Include 'hershey.cmb'

	Data POW2(0)/0/
	Data IOFF/0/ 
	Data IUP/3/, IDOWN/2/ 
	Data FACTOR/0.75/, SUPSUB/0.50,-0.50/
	Data RAD/1.745329252e-2/

c  IXTRCT gets NBITS from IWORD starting at the NSTART bit from the right

	IXTRCT(NSTART,NBITS,IWORD) = Mod (IWORD / (POW2(NSTART - NBITS)),
     1				   POW2(NBITS)) + ((1 - ISIGN (1,IWORD)) / 2) * 
     2				   (POW2(NBITS) - MIN0 (1,MOD ( - IWORD,
     3				   POW2(NSTART - NBITS))))

	If (POW2(0) .eq. 0) Then
	    POW2(0) = 1
	    Do 5 I = 1, 30
5		POW2(I) = 2 * POW2(I-1)
	    Endif

	YOFF = 0.0
	SI = Sin (RAD * THETA)
	CO = Cos (RAD * THETA)
	SCALE = HEIGHT / 21.
	If (SCALE .eq. 0.0) Return
	If (X .lt. 0.) Then
	    XI = XO
	Else
	    XI = X
	Endif
	If (Y .lt. 0.) Then
	    YI = YO
	Else
	    YI = Y
	Endif

c  Plot a single special centered symbol

	If (NTEXT .lt. 0) Then
	    If (NTEXT .lt.  -1) Call FSTYLUS (IMAGE,XI,YI,IDOWN)
	    IA = Ichar (TEXT(1:1)) + 1
	    IS = ISSTAR(IA)
	    IB = 30
20	    IPEN = IXTRCT(IB,3,SSYMBC(IS))
	    If (IPEN .eq. 0) Then
		Call FSTYLUS (IMAGE,XI,YI,IUP)
		XI = XI + 20.0 * CO
		YI = YI + 20.0 * SI
		XO = XI
		YO = YI
		Return
		Endif
	    IX = IXTRCT(IB - 3,6,SSYMBC(IS))
	    IY = IXTRCT(IB - 9,6,SSYMBC(IS))
	    XX = SCALE * (IX - 32)
	    YY = SCALE * (IY - 32)
	    Call FSTYLUS (IMAGE,XI+(XX*CO)-(YY*SI), YI+(XX*SI)+(YY*CO), IPEN)
	    IB = 45 - IB
	    If (IB .eq. 30)IS = IS + 1
	    Go to 20

c  Plot a single simplex Roman character

	ElseIf  (NTEXT .eq. 0)Then
	    ISAV = IOFF
	    IOFF = 0
	    Call FCHRCOD(TEXT,1)
	    IOFF = ISAV
	    IS = ISTART(ICHR(1))
	    IB = 30
40	    IPEN = IXTRCT(IB,3,SYMBCD(IS))
	    If (IPEN .eq. 0) Then
		XI = XI + CO * SCALE * WIDTH(ICHR(1))
		YI = YI + SI * SCALE * WIDTH(ICHR(1))
		XO = XI
		YO = YI
		Return
		Endif
	    IX = IXTRCT(IB - 3,6,SYMBCD(IS))
	    IY = IXTRCT(IB - 9,6,SYMBCD(IS))
	    XX = (IX - 10) * SCALE
	    YY = (IY - 11) * SCALE
	    Call FSTYLUS (IMAGE,XI+(CO*XX)-(SI*YY),YI+(CO*YY)+(SI*XX),IPEN)
	    IB = 45 - IB
	    If (IB .eq. 30)IS = IS + 1
	    Go to 40

c  Plot a character string.

	Else

c  First find pointer array ICHR containing the starts of characters - 
c  but only if JUST1 and JUST2 are not 1, when ICHR is assumed
c  correctly transmitted through Common  / AJUST / .

	    If ((JUST1 .ne. 1) .or. (JUST2 .ne. 1)) Then
		N = NTEXT
		Call FCHRCOD(TEXT,N)
		Endif
	    JUST2 = 2
	    OLDWID = 0.0
	    L = 1
	    RSCALE = SCALE

c  Plot each character

	    Do 100 I = 1,NCHR
		IC = ICHR(I)

c	Plot a space

		If (IC .eq. 1000)Then
		    XI = XI + 20. * RSCALE * CO
		    YI = YI + 20. * RSCALE * SI
		    XO = XI
		    YO = YI
		    Call FSTYLUS (IMAGE,XI,YI,IUP)

c	Begin super-scripting or sub-scripting

		Elseif  ((IC .eq. 1001).OR.(IC .eq. 1002))Then
		    RAISE(L) = SUPSUB(IC - 1000) * HEIGHT * RSCALE / SCALE
		    RSCALE = FACTOR * RSCALE
		    YOFF = RAISE(L) + YOFF
		    L = L + 1

c	End super/sub-scripting

		Elseif  (IC .eq. 1003)Then
		    RSCALE = RSCALE / FACTOR
		    L = L - 1
		    YOFF = YOFF - RAISE(L)

c	Backspace - use the width of the previous letter in OLDWID.

		Elseif  (IC .eq. 1004)Then
		    XI = XI  -  CO * OLDWID
		    YI = YI  -  SI * OLDWID
		    XO = XI
		    YO = YI

c	Plot a single symbol

		Else
		    IS = ISTART(IC)
		    IB = 30
70		    IPEN = IXTRCT(IB,3,SYMBCD(IS))
		    If (IPEN .eq. 0)Then
			XI = XI + CO * RSCALE * WIDTH(IC)
			YI = YI + SI * RSCALE * WIDTH(IC)
			XO = XI
			YO = YI
			OLDWID = WIDTH(IC) * RSCALE
			Go to 100
			Endif
		    IX = IXTRCT(IB - 3,6,SYMBCD(IS))
		    IY = IXTRCT(IB - 9,6,SYMBCD(IS))
		    XX = (IX - 10) * RSCALE
		    YY = (IY - 11) * RSCALE + YOFF
		    Call FSTYLUS (IMAGE,XI+(CO*XX)-(SI*YY),YI+(CO*YY)+(SI*XX),IPEN)
		    IB = 45 - IB
		    If (IB .eq. 30)IS = IS + 1
		    Go to 70
		Endif

100	    Continue
	Endif

	Return
	End


	Subroutine FCHRCOD (TEXT,NTEXT)

c  Given text string in TEXT, NTEXT Characters returns ICHR containing
c  NCHR symbol numbers or codes for <space> (1000), <begin superscripting>
c  (1001), <begin subscripting> (1002), <end super/sub-scripting> (1003),
c  <backspace> (1004), <vector> (1005), or <hat> (1006).
c  Change of font commands are decoded and executed internally.

	Character*(*) TEXT
	Integer*4 NTEXT

	Integer*4 N,NT,K,J,IC,NUMBER,L,IGOFF,IGR,ICO,IG,IB

	Common/FOFFSET/  IOFF,JUST1,JUST2
	Integer*4 IOFF,JUST1,JUST2

	Common/AJUST/ NCHR,ICHR(350)
	Integer*4 NCHR,ICHR

	Character*1 BACKSLASH

	Integer IRLU(95),IILU(95),IGLU(26)

c  IRLU is a look-up table for ROMAN characters arranged by
c  integer value for the ASCII character set with an
c  offset to remove the 31 nonprinting control characters.
c  IRLU returns with the symbol number or, if no symbol exists,
c  the code for space.

	Data IRLU / 1000,416,428,411,72,418,419,432,67,68,69,63,70,
     1	    64,71,65,53,54,55,56,57,58,59,60,61,62,414,415,
     2	    385,66,386,417,407,1,2,3,4,5,6,7,8,9,10,11,12,13,
     3	    14,15,16,17,18,19,20,21,22,23,24,25,26,409,1000,
     4	    410,408,1000,1000,27,28,29,30,31,32,33,34,35,36,
     5	    37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,
     6	    405,427,406,424 / 

c  IILU is a look-up table for italic characters only.  It is
c  identical to IRLU with four italic special symbols substituted
c  for regular ones.

	Data IILU / 1000,422,1000,411,72,418,419,1000,67,68,69,63,70,
     1	    64,71,65,53,54,55,56,57,58,59,60,61,62,420,421,
     2	    385,66,386,423,407,1,2,3,4,5,6,7,8,9,10,11,12,13,
     3	    14,15,16,17,18,19,20,21,22,23,24,25,26,409,1000,
     4	    410,1000,1000,1000,27,28,29,30,31,32,33,34,35,36,
     5	    37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,
     6	    405,427,406,424 / 

c  IGLU is a look-up table for Greek characters arranged by the
c  integer value of their Roman expression with A = 1, B = 2, etc.
c  Ambiguous cases give 25 for Epsilon or Eta, 26 for Omega or
c  Omicron, 27 for Phi, Pi, or Psi, and 28 for Tau or Theta.  Additional
c  letters must be checked for these cases.  A value of 50 is returned
c  for those Roman letters which have no corresponding Greek letter.

	Data IGLU / 1,2,22,4,25,50,3,50,9,50,10,11,12,13,26,27,50,17,18,
     1		 28,20,50,50,14,50,6 / 

	BACKSLASH = '|'

c Find length of string with blanks trimmed from right end

	Do 10 N = NTEXT,1, - 1
10	    If (TEXT(N:N) .ne. ' ')Go to 15
	NCHR = 0
	Return

15	NT = N

c  Scan text character by character
	K = 1
	J = 1

c  K is current address of character in text
c  J is index of next symbol code in ICHR

20	If (K .gt. N)Then
	    NCHR = J - 1
	    Return
	    Endif

c  Roman character or keyboard symbol

	If (TEXT(K:K) .ne. BACKSLASH)Then

c  Check for closing curly bracket; if found, return 1003

	    If (TEXT(K:K) .eq. '}')Then
		ICHR(J) = 1003
		J = J + 1
		K = K + 1
		Go to 20
		Endif

c  Ichar returns integer ASCII value of character
c  offset by nonprinting characters to get entry in look-up table

	    IC = Ichar (TEXT(K:K)) - Ichar (' ') + 1

c  Nonprinting control character = error return

	    If (IC .le. 0) Then
		ICHR(J) = 1000

c  Not italic font

	    Elseif (IOFF .ne. 240) Then
		ICHR(J) = IRLU(IC)

c  Italic font

	    Else
		ICHR(J) = IILU(IC)
	    Endif

c  Add offset for font if not a special symbol

	    If (ICHR(J) .lt. 385)ICHR(J) = ICHR(J) + IOFF
		J = J + 1
		K = K + 1
		Go to 20

c  Backslash found
c  Check next four characters for four-digit number

	    Else
		K = K + 1
		Read(TEXT(K:K + 3),25,ERR = 50)NUMBER
25		Format(I4)

c  Number found; check its validity

		IC = NUMBER - 1000

c  Valid symbol code

		If ((IC.GT.0) .and. (IC .lt. 433))Then
		    ICHR(J) = IC

c  Valid command code

		ElseIf  ((IC.GT.999) .and. (IC .lt. 1004))Then
		    ICHR(J) = IC

c Not recognized; error returnn

		Else
		    ICHR(J) = 1000
		Endif
		J = J + 1

c  Move beyond closing backslash; ignore extra characters
c  Function Index returns offset of second substring in first, and
c  returns 0 if substring not found.

		L = Index (TEXT(K:NT),BACKSLASH)
		If (L .eq. 0)Then
		    K = NT + 1
		Else
		    K = K + L
		Endif
		Go to 20
50		Continue

c	Not a number:
c	Check for font change command

c	Simplex font
		If (TEXT(K:K + 2) .eq. 'SIM'.OR.TEXT(K:K + 2) .eq. 'sim')Then
		    IOFF = 0

c	Complex font
		Elseif (TEXT(K:K + 1) .eq. 'CO'.OR.TEXT(K:K + 1) .eq. 'co')Then
		    IOFF = 120

c	Italic font
		Elseif (TEXT(K:K + 1) .eq. 'IT'.OR.TEXT(K:K + 1) .eq. 'it')Then
		    IOFF = 240

c	Duplex font
		Elseif  (TEXT(K:K + 1) .eq. 'DU'.OR.TEXT(K:K + 1) .eq. 'du')Then
		    IOFF = 312

c	Found the back-space code
		Elseif (TEXT(K:K + 1) .eq. 'BS'.OR.TEXT(K:K + 1) .eq. 'bs') Then
		    ICHR(J) = 1004
		    J = J + 1
		    K = K + 3
		    Go to 20

c	Check for super/sub-script command

c	Begin superscripting

		Elseif (TEXT(K:K + 3) .eq. 'SUP{'.OR.TEXT(K:K + 3) .eq. 'sup{')Then
		    ICHR(J) = 1001
		    J = J + 1
		    K = K + 4
		    Go to 20

c	Begin subscripting

		Elseif  (TEXT(K:K + 3) .eq. 'SUB{'.OR.TEXT(K:K + 3) .eq. 'sub{')Then
		    ICHR(J) = 1002
		    J = J + 1
		    K = K + 4
		    Go to 20

c	Greek character or invalid character

		Else
		    IC = Ichar (TEXT(K:K))
		    IGOFF = MIN0(IOFF, 120)
		    If (IOFF .eq. 312)IGOFF = 0

c	Upper case
		    If ((IC .ge. Ichar ('A')) .and. (IC .le. Ichar ('Z')))Then
			IGR = 72
			ICO = Ichar ('A') - 1
c	Lower case
		    ElseIf ((IC .ge. Ichar ('a')) .and. (IC .le. Ichar ('z')))Then
			IGR = 96
			ICO = Ichar ('a') - 1

c	Not a letter - error return
		    Else
			ICHR(J) = 1000
			J = J + 1
			L = Index(TEXT(K:NT),BACKSLASH)
			If (L .eq. 0)Then
		  	    K = NT + 1
			Else
			    K = K + L
			Endif
			Go to 20
		    Endif

c	Look up the character

		    IG = IGLU(IC - ICO)

c	Unambiguous Greek letter
		    If (IG .lt. 25) Then
			ICHR(J) = IG + IGR + IGOFF

c	Epsilon or Eta
		    Elseif  (IG .eq. 25) Then
			IB = Ichar (TEXT(K + 1:K + 1)) - ICO

c		Epsilon
			If (IB .eq. 16) Then
			    ICHR(J) = 5 + IGR + IGOFF
c		Eta
		 	Elseif  (IB .eq. 20)Then
			    ICHR(J) = 7 + IGR + IGOFF

c		Not a Greek character -- Error return
			Else
			    ICHR(J) = 1000
			Endif

c	Omega or Omicron
		    Elseif  (IG .eq. 26)Then
			IB = Ichar (TEXT(K + 1:K + 1)) - ICO

c		Not a Greek character -- Error return
			If (IB .ne. 13)Then
			    ICHR(J) = 1000

			Else
			    IC = Ichar (TEXT(K + 2:K + 2)) - ICO

c		Omega
			    If (IC .eq. 5)Then
				ICHR(J) = 24 + IGR + IGOFF
c		Omicron
			    Elseif  (IC .eq. 9)Then
				ICHR(J) = 15 + IGR + IGOFF

c		Not a Greek character -- Error return
			    Else
				ICHR(J) = 1000
			    Endif
			Endif

c	Phi, Pi, or Psi
		    ElseIf  (IG .eq. 27)Then
			IB = Ichar (TEXT(K + 1:K + 1)) - ICO

c		Phi
			If (IB .eq. 8) Then
			    ICHR(J) = 21 + IGR + IGOFF
c		Pi
			Elseif  (IB .eq. 9) Then
			    ICHR(J) = 16 + IGR + IGOFF
c		Psi
			Elseif  (IB .eq. 19)Then
			    ICHR(J) = 23 + IGR + IGOFF
c		Not a Greek character -- Error return
			Else
			    ICHR(J) = 1000
			Endif

c	Tau or Theta
		    Elseif  (IG .eq. 28)Then
			IB = Ichar (TEXT(K + 1:K + 1)) - ICO
c		Tau
			If (IB .eq. 1)Then
			    ICHR(J) = 19 + IGR + IGOFF
c		Theta
			ElseIf (IB .eq. 8)Then
			    ICHR(J) = 8 + IGR + IGOFF
c		Not a Greek character -- Error return
			Else
			    ICHR(J) = 1000
			Endif

c	Not a Greek character -- Error return
		    Else
			ICHR(J) = 1000
		    Endif

		    J = J + 1
		Endif

		L = Index (TEXT(K:NT),BACKSLASH)
		If (L .eq. 0)Then
		    K = NT + 1
		Else
		    K = K + L
		Endif
		Go to 20
	    Endif

	End


	Subroutine FJUSTFY (S, HEIGHT, TEXT, NTEXT)

c  Given the text string TEXT with NTEXT characters, HEIGHT,
c  text string TEXT with NTEXT characters.  This routine
c  gives 4 distances in inches, all from the left end of the string  - 
c  S(1)  to the left edge of the 1st nonblank character
c  S(2)  to the center of the string, blanks removed from the ends
c  S(3)  to the right edge of the last nonblank character
c  S(4)  to the right edge of the last character of the string.
c$$$$ Calls FCHRCOD

	Real*4 S(4)
	Real*4 HEIGHT
	Character*(*) TEXT
	Integer*4 NTEXT

	Integer*4 NTXT,LEAD,I,L
	Real*4 SCALE,OLDWID

	Common/FWID/  WIDTH
	Real*4 WIDTH(432)

	Common/FOFFSET/  IOFF,JUST1,JUST2
	Integer*4 IOFF,JUST1,JUST2

	Common/AJUST/ NCHR,ICHR(350)
	Integer*4 NCHR,ICHR

	Integer*4 IPOWER(3)
	Real*4 FACTOR

	Data IPOWER/1,1,-1/
	Data FACTOR/0.75/ 

	NTXT = NTEXT
	SCALE = HEIGHT / 21.0

c  Translate Integer string into Character variable, then get pointers
c  into the array ICHR.

	Call FCHRCOD(TEXT,NTXT)

c  Count leading blanks.

	Do 1100 LEAD = 1,NCHR
1100	    If (ICHR(LEAD) .ne. 1000) Go to 1110
	LEAD = NTXT
1110	S(1) = 20.0 * SCALE * (LEAD - 1)
	S(3) = S(1)

c  Sum the widths of the remaining text, recalling that trailing blanks
c  were lopped off by FCHRCOD.

	OLDWID = 0.0
	Do 1200 I = LEAD,NCHR
	    L = ICHR(I)
	    If  (L .lt. 1000) Then
		OLDWID = WIDTH(L) * SCALE
		S(3) = S(3)  +  OLDWID
		Endif
	    If (L .eq. 1000) S(3) = S(3) + 20.0 * SCALE
	    If (L .ge. 1001 .and. L .le. 1003)
     1		SCALE = SCALE * FACTOR ** IPOWER(L - 1000)
1200	    If (L .eq. 1004) S(3) = S(3) - OLDWID

c  Add on width of surplus trailing blanks.

	S(4) = S(3) + 20.0 * SCALE * (NTXT - NCHR)

c  Find center of nonblank text.

	S(2) = (S(1) + S(3)) / 2.0
	JUST2 = 1
	Return
	End

c Feb 25 1993	Drop unused variable pi

c Aug 26 1996	Move all Data statements to after declarations
