c*** File fitslib/fitsread.f
c*** February 25, 1993
c*** By Doug Mink

c--- Read FITS header and data after opening a data file
c    Note settings of /HEADER/ common.

	Subroutine FITSREAD (FNAME,HSTRING,NR,IMAGE,NBMAX,NBYTES)

	Character*(*) FNAME
c				File name
	Character*(*) HSTRING
c				Header string (returned)
	Integer*4 NR
c				Number of characters read (returned)
c				Value of 0 returned if unsuccessful
	Integer*4 IMAGE(*)
c				FITS image (returned)
	Integer*4 NBMAX
c				Number of bytes in image array
	Integer*4 NBYTES
c				Number of image bytes read (returned)
c				Value of 0 returned if unsuccessful

	Integer*4 IFIL
c				UNIX file from which to read

	Character NULL*1
	Logical BSWAP
	Integer*4 LHEAD,LMAX,LFNAM,I,I0,LREC,IEND,LHEAD0,IER
	Integer*4 BITPIX,NAXIS,NAXIS1,NAXIS2,NAXIS3,NBR0,NBPIX

	Include 'header.cmb'

	NULL = Char (0)
	Call ESET
	Call VLEN (FNAME,LFNAM)

c  Initialize header string to NULL

	LMAX = Len (HSTRING)
	Call ZSTRING (HSTRING)

c  Open data file, returning if not found

	IFIL = -2
	NR = 0
	Call OPENR (IFIL,FNAME)
	IF (IFIL .lt. 3) Return

c  Read first block of FITS header

	LHEAD = 2880
	LREC = 2880
	Call SGETC (IFIL,LREC,HSTRING,NR)
50	If (NR .lt. 2) Then
	   Write(*,60) NR,FNAME(1:LFNAM)
60	   Format('FITSREAD:',I3,' bytes in file "',A,'"')
	   NR = 0
	   Return
	   Endif

c  If END is found, pad rest of header string with NULLs

	I0 = 1
	Call HSEARCH (HSTRING,'END',IEND)
	If (IEND .gt. 0) Then
	    I0 = IEND + 80
	    Do 100 I = I0, LMAX
100		HSTRING(I:I) = NULL

c  If END is not found, read next block

	Else
	    LHEAD0 = LHEAD + 1
	    LHEAD = LHEAD + LREC
	    If (LHEAD .gt. LMAX) Then
		Write(*,105) FNAME(1:LFNAM)
105		Format('FITSREAD: No header END in file "',A,'"')
		NR = 0
		Return
		Endif
	    Call SGETC (IFIL,LREC,HSTRING(LHEAD0:LHEAD),NR)
	    Go to 50
	Endif

	If (VERBOSE) Then
	    Write(*,120) LHEAD,FNAME(1:LFNAM)
120	    Format('FITSREAD:',I5,'-byte header read from file "',A,'"')
	    Endif
	NR = LHEAD

c  Compute number of bytes in image

	Call HGETI4 (HSTRING,'BITPIX',BITPIX)
	Call HGETI4 (HSTRING,'NAXIS',NAXIS)
	Call HGETI4 (HSTRING,'NAXIS1',NAXIS1)
	NBPIX = BITPIX / 8
	If (NBPIX .lt. 0) NBPIX = -NBPIX
	NBR0 = NAXIS1 * NBPIX
	If (NAXIS .gt. 1) Then
	    Call HGETI4 (HSTRING,'NAXIS2',NAXIS2)
	    NBR0 = NBR0 * NAXIS2
	Else
	    NAXIS2 = 1
	Endif
	If (NAXIS .gt. 2) Then
	    Call HGETI4 (HSTRING,'NAXIS3',NAXIS3)
	    NBR0 = NBR0 * NAXIS3
	Else
	    NAXIS3 = 1
	Endif

c  Check limits

	If (NBR0 .gt. NBMAX) Then
	    Write(*,130) NBR0,NBMAX,FNAME(1:LFNAM)
130	    Format('*** FITSREAD: ',I7,' > ',I7,' bytes in file "',A,'"')
	    NBR0 = NBMAX
	    Endif

c  Read in data

	Call SGET (IFIL,NBR0,IMAGE,NBYTES)
	If (NBR0 .gt. NBYTES) Then
	    Write(*,135) NBYTES,NBR0,FNAME(1:LFNAM)
135	    Format('*** FITSREAD: ',I8,' < ',I8,' bytes in file "',A,'"')
	    Endif
	If (VERBOSE) Then
	    Write(*,140) NAXIS1,NAXIS2,NAXIS3,NBPIX,NBYTES,FNAME(1:LFNAM)
140	    Format('FITSREAD:',I5,' x',I5,' x',I5,' /',I3,' =',I8,
     1		   ' bytes of data read from file "',A,'"')
	    Endif
	Call SCLOSE (IFIL,IER)

c  Swap data bytes to VAX or Decstation integer format

	Call BYTEORDER (BSWAP)
	If (BSWAP) Then
	    If (BITPIX .eq. -32) Then
		Call SSWAP4 (IMAGE,NBYTES)
	    Elseif (BITPIX .eq. 32) Then
		Call SSWAP4 (IMAGE,NBYTES)
	    Elseif (BITPIX .eq. 16) Then
		Call SSWAP2 (IMAGE,NBYTES)
	    Elseif (BITPIX .eq. -64) Then
		Call SSWAP2 (IMAGE,NBYTES)
	    Endif
	    If (VERBOSE) Then
		Write(*,150) NBYTES,FNAME(1:LFNAM)
150		Format('FITSREAD:',I8,' bytes swapped from file "',A,'"')
		Endif
	    Endif

	Return
	End

c Sep 25 1991   Take care of negative BITPIX for floating point images

c Jan 10 1992	Make separate Sun and DEC versions of this subroutine

c Feb 25 1993	Call BYTEORDER to determine whether or not to swap bytes
