c*** File fitslib/fitswchar.f
c*** January 17, 1997
c*** By Doug Mink

c--- Plot character string in file at given location using Hershey fonts

	Subroutine FITSWCHAR (HEADER,IMAGE,IX,IY,STRING)

	Character*(*) HEADER
c			FITS header for image
	Integer*2 IMAGE(*)
c			FITS image
	Integer*4 IX,IY
c			Image coordinates to start
c			(lower left of first character)
c			* if IX is <0, center string at abs (IX)
c			* if IY is <0, center characters at abs (IY)
c			* If both are =0, no plotting is done and CR LF is added
	Character STRING*(*)
c				String to print (NULL or '$' terminated)

	Integer*4 NCHARS
c				Number of characters to be plotted

	Character*20 LOGFILE
	Character*100 TEMP
	Integer*4 NAXIS,NAXIS2
	Real*4 DX,DY,X,Y
	Include 'header.cmb'

	Include 'fitschar.cmb'
	Data X0/0./, Y0/0./,HEIGHT/20./,THICK/2./,THETA/0./,CVAL/0/

	Call HGETI4 (HEADER,'BITPIX',BITPIX)
	Call HGETI4 (HEADER,'NAXIS',NAXIS)
        Call HGETI4 (HEADER,'NAXIS1',NAXIS1)
        If (NAXIS .gt. 1) Then
            Call HGETI4 (HEADER,'NAXIS2',NAXIS2)
	Else
	    NAXIS2 = 1
        Endif

c  Find length of string

	Call PADNULL (STRING)
	Call VLEN (STRING,NCHARS)

c  Return if no characters in string

	If (NCHARS .le. 0) Return

c  Return if coordinates not set

	If ((IX .eq. 0) .and. (IY .eq. 0)) Return

	LOGFILE = 'fitswchar.log'
	Call PADNULL (LOGFILE)
	Call PLOGSET (LOGFILE,.false.)

c  Find size of string

	DX = 0.
	DY = 0.

c  Center string in X or Y if requested

	X = Float (IX)
	Y = Float (IY)
	If ((IX .lt. 0) .or. (IY .lt. 0)) Then
	    Call FITSFST (STRING,DX,DY)
	    If (IX .lt. 0) X = Float (-IX) - (DX * 0.5)
	    If (IY .lt. 0) Y = Float (-IY) - (DY * 0.5)
	    If (VERBOSE) Then
		Write(TEMP,*) 'FITSWCHAR: Centering',IX,IY,DX,DY,X,Y
		Call PLOG (TEMP)
		Endif
	    Endif
	If (X .lt. 0) X = 0.
	If (Y .lt. 0) Y = 0.
	
c  Plot string using Hershey font set

	Call FITSHERSH (IMAGE,X,Y,HEIGHT,STRING,THETA,NCHARS)

	Return
	End


c--- Translate plotting calls from HERSHEY, called by program above

	Subroutine FSTYLUS (IMAGE, X1, Y1, IUP)

	Integer*2 IMAGE(*)
c			FITS image array
	Real*4 X1,Y1
c			X, Y coordinates from HERSHEY
	Integer*4 IUP
c			=3 pen up, =2 pen down, else up

	Integer*4 IX, IY, IPEN,IPIX,NPIX,OFFSET,IVAL,IT,NT
	Real*4 X,Y,XFRAC,YFRAC,THICKOFF
	Real*4 RVAL
	Real*8 DVAL
	Character*100 STRING

	Include 'header.cmb'
	Include 'fitschar.cmb'

	IPEN = 0
	If (IUP .eq. 2) IPEN = 1
	IVAL = CVAL
	NT = Nint (THICK)
	THICKOFF = (THICK - 1.) * 0.5

	IX = Nint (X1)
	IY = Nint (Y1)
	If (VERBOSE) Then
	    OFFSET = (Nint (Y1) - 1) * NAXIS1 + Nint (X1)
	    Write(STRING,10) IPEN,X1,Y1,X0,Y0,NAXIS1,OFFSET,NT
10	    Format('FSTYLUS:',I2,2F7.1,' from',2F7.1,' *',I5,' =',I9,I2,'$')
	    Call PLOG (STRING)
	    Endif
	If (IPEN .eq. 1) Then
	    If ((X0 .gt. 0) .and. (Y0 .gt. 0)) Then
		NPIX = Nint (Sqrt ((X1-X0)*(X1-X0) + (Y1-Y0)*(Y1-Y0)) * 2.)
		If (NPIX .lt. 1) NPIX = 1
		XFRAC = (X1 - X0) / NPIX
		YFRAC = (Y1 - Y0) / NPIX
		X = X0 - XFRAC - THICKOFF
		Y = Y0 - YFRAC - THICKOFF
		Do 100 IPIX = 1, NPIX
		    X = X + XFRAC
		    Y = Y + YFRAC
		    IX = Nint (X)
		    IY = Nint (Y)
		    OFFSET = ((IY - 1) * NAXIS1) + IX
		    If (VERBOSE) Then
			Write(STRING,20) IPIX,NPIX,IX,IY,OFFSET,NT
20			Format('FSTYLUS:',I3,'/',I3,2I5,' =',I9,I2,'$')
			Call PLOG (STRING)
			Endif
		    Do 90 IT = 1, NT
			If (BITPIX .eq. 8) Then
			    Call SVSET1C (IVAL,IMAGE,OFFSET,NT)
			Elseif (BITPIX .eq. 16) Then
			    Call SVSET2C (IVAL,IMAGE,OFFSET,NT)
			Elseif (BITPIX .eq. 32) Then
			    Call SVSET4C (IVAL,IMAGE,OFFSET,NT)
			Elseif (BITPIX .eq. -32) Then
			    RVAL = IVAL
			    Call SVSETRC (RVAL,IMAGE,OFFSET,NT)
			Elseif (BITPIX .eq. -64) Then
			    DVAL = IVAL
			    Call SVSETDC (DVAL,IMAGE,OFFSET,NT)
			Endif
90			OFFSET = OFFSET + NAXIS1
100		    Continue
	    Else
		IX = Nint (X1)
		IY = Nint (Y1)
		OFFSET = ((IY - 1) * NAXIS1) + IX
		If (VERBOSE) Then
		    Write(STRING,20) IPEN,IX,IY,OFFSET,NT
		    Call PLOG (STRING)
		    Endif
		Do 190 IT = 1, NT
		    If (BITPIX .eq. 8) Then
			Call SVSET1C (IVAL,IMAGE,OFFSET,NT)
		    Elseif (BITPIX .eq. 16) Then
			Call SVSET2C (IVAL,IMAGE,OFFSET,NT)
		    Elseif (BITPIX .eq. 32) Then
			Call SVSET4C (IVAL,IMAGE,OFFSET,NT)
		    Elseif (BITPIX .eq. -32) Then
			RVAL = IVAL
			Call SVSETRC (RVAL,IMAGE,OFFSET,NT)
		    Elseif (BITPIX .eq. -64) Then
			DVAL = IVAL
			Call SVSETDC (DVAL,IMAGE,OFFSET,NT)
		    Endif
190		    OFFSET = OFFSET + NAXIS1
	    Endif
	    Endif
	
	X0 = X1
	Y0 = Y1

	Return
	End


c--- Set height of characters for Hershey fonts

	Subroutine FITSCSIZ (IHT)

	Integer*4 IHT
c			Height of characters in pixels

	Include 'fitschar.cmb'

	HEIGHT = IHT
	Return
	End


c--- Set line thickness of characters for Hershey fonts

	Subroutine FITSTHICK (ITH)

	Integer*4 ITH
c			Thickness of character lines in pixels

	Include 'fitschar.cmb'

	THICK = ITH
	Return
	End


c--- Set line thickness of characters for Hershey fonts

	Subroutine FITSCVAL (IVAL)

	Integer*4 IVAL
c			Value to which character pixels are set

	Include 'fitschar.cmb'

	CVAL = IVAL
	Return
	End


c--- Set font for Hershey characters

	Subroutine FITSFONT (HEADER,IMAGE,CFONT)

	Character*(*) HEADER
c			FITS header for image
	Integer*2 IMAGE(*)
c			FITS image
	Character*1 CFONT
c			Font for characters (Simplex, Duplex, Complex, Italic)

	Integer*4 IX,IY

	IX = 1
	IY = 1
	If ((CFONT .eq. 'C') .or. (CFONT .eq. 'c')) Then
	    Call FITSWCHAR (HEADER,IMAGE,IX,IY,'|complex|$')
	Elseif ((CFONT .eq. 'D') .or. (CFONT .eq. 'd')) Then
	    Call FITSWCHAR (HEADER,IMAGE,IX,IY,'|duplex|$')
	Elseif ((CFONT .eq. 'I') .or. (CFONT .eq. 'i')) Then
	    Call FITSWCHAR (HEADER,IMAGE,IX,IY,'|italic|$')
	Elseif ((CFONT .eq. 'S') .or. (CFONT .eq. 's')) Then
	    Call FITSWCHAR (HEADER,IMAGE,IX,IY,'|simplex|$')
	Endif

        Return
        End


c--- Find character string length and height when using Hershey fonts

	Subroutine FITSFST (STRING,DX,DY)

	Character STRING*(*)
c				String to print (NULL or '$' terminated)
	Real*4 DX,DY
c				Extent of string in screen x and y coordinates

	Integer*4 NCHARS
c				Number of characters to be plotted
	Real*4 S(4)
c				String lengths returned from FJUSTFY
	Real*4 TH,STH,CTH
c				Rotation angle in radians

	Include 'fitschar.cmb'

c  Find length of string

	Call VLEN (STRING,NCHARS)

20	If (NCHARS .le. 0) Return

c  Scale factor for converting to Tektronix screen coordinates

	TH = THETA * 1.745329e-2
	STH = Sin (TH)
	CTH = Cos (TH)
	Call FJUSTFY (S,HEIGHT,STRING,NCHARS)
	DX = S(3) * CTH + HEIGHT * STH
	DY = S(3) * STH + HEIGHT * CTH
	
	Return
	End

c Mar 27 1991	Program written

c Feb 25 1993	Drop unused variable null

c Jan 17 1997	Add support for floating point images
