c*** File fitslib/hget.f
c*** September 18, 2003
c*** By Doug Mink

c--- Subroutines to extract values for variables from FITS header string

c	HGET	(HSTRING,VNAME,ITYPE,IVAL) returns any type
c	HGETI2	(HSTRING,VNAME,IVAL) returns Integer*2 IVAL
c	HGETI4	(HSTRING,VNAME,IVAL) returns Integer*4 IVAL
c	HGETR4	(HSTRING,VNAME,RVAL) returns Real*4 RVAL
c	HGETRA	(HSTRING,VNAME,RA) returns Real*8 RA in radians
c	HGETDEC	(HSTRING,VNAME,DEC) returns Real*8 DEC in radians
c	HGETANG	(HSTRING,VNAME,DEC) returns Real*8 angle in radians
c	HGETR8	(HSTRING,VNAME,DVAL) returns Real*8 DVAL
c	HGETJD	(HSTRING,JD) returns Real*8 Julian Date
c	HGETDATE (HSTRING,VNAME,YR,MN,DAY,UT) returns date and optional time
c	HGETL	(HSTRING,VNAME,LVAL) returns Logical LVAL
c	HGETC	(HSTRING,VNAME,CVAL) returns Character string CVAL
c		(removes quotes from character strings)
c	HGETCOM (COMMENT) Return comment for last parameter gotten from string


c--- HGET  Extract value for variable from FITS header string (any type)

	Subroutine HGET (HSTRING,VNAME,ITYPE,IVAL)

	Character*(*) HSTRING
c		Character string containing FITS-style header information
c		in the format <VNAME>= <value> {/ <comment>}
c		Unlike FITS, however, lines may be of arbitrary length
c		terminated by nulls, carriage returns or linefeeds,
c		although FITS' 80-column format is supported, too.

	Character*(*) VNAME
c		Character string containing the name of the variable
c		to be returned.  HGET searches for a line beginning
c		with this string.  If "[n]" or ",n" is present, the n'th
c		 token in the value is returned.
c		(THE FIRST 8 CHARACTERS OF A VARIABLE NAME MUST BE UNIQUE)

	Integer*4 ITYPE
c		Data type
c		  first ':' treated as '.', later ':'s and '.'s ignored.
c		  illegal characters, overflows, and underflows trapped. 
c		-1n: Integer n bytes long (Integer*n)
c		-2n: Floating point number n bytes long (Real*n)
c		-3n: Logical variable n bytes long (Logical*n) (t or f)
c		-4n: Logical variable n bytes long (Logical*n) (y or n)
c		1-99: Length of Character string returned, padded with spaces
c		101-199: Length (+100) of Character string returned
c		        padded with nulls
c		201-236: number (+200) of single characters
c			returned in array of 36 (A-Z,0-9) Integer*2
c			with array positions corresponding to letter
c			or number entered set to one.  Array is not
c			zeroed on entry.
c			Lower case letters are treated as upper case.
c		301-399: Length (+300) of Character string returned
c		        padded with nulls
c			Lower case letters are converted to upper case.

c	(Integer,Real) IVAL
c		Address for returned number, character, array, or string
c		If a Character string is to be returned, call with
c		Equivalence'd Integer; otherwise, Fortran adds another
c		invisible string length to the calling parameter list
c		and messes up everything.

	Character*30 VALUE
c		Character string containing value from header

	Integer*2 IVAL(*)
	Integer*4 LVAL,IER
	Include 'header.cmb'

c  Get value and comment from header string

	Call HGETC (HSTRING,VNAME,VALUE)

c  Translate value from ASCII to binary

	If (FOUND) Then
	    Call VLEN (VALUE,LVAL)
	    Call ASCBIN (LVAL,VALUE,ITYPE,IVAL,IER)
	    Endif

c  Return to calling program

	Return
	End


c--- HGETI4  Extract Integer*4 value for variable from FITS header string

	Subroutine HGETI4 (HSTRING,VNAME,IVAL)

	Character*(*) HSTRING
c		Character string containing FITS-style header information
	Character*(*) VNAME
c		Character string containing the name of the variable
	Integer*4 IVAL

	Character*30 VALUE
	Integer*4 LVAL,IER
	Include 'header.cmb'

c  Get value and comment from header string

	Call HGETC (HSTRING,VNAME,VALUE)
	Call HCLEAN (VALUE)

c  Translate value from ASCII to binary

	If (FOUND) Then
	    Call VLEN (VALUE,LVAL)
	    Call ASCBINI4 (LVAL,VALUE,IVAL,IER)
	    Endif
	Return
	End


c--- HGETI2  Extract Integer*2 value for variable from FITS header string

	Subroutine HGETI2 (HSTRING,VNAME,IVAL)

	Character*(*) HSTRING
c		Character string containing FITS-style header information
	Character*(*) VNAME
c		Character string containing the name of the variable
	Integer*2 IVAL

	Character*30 VALUE
	Integer*4 LVAL,IER
	Include 'header.cmb'

c  Get value and comment from header string

	Call HGETC (HSTRING,VNAME,VALUE)
	Call HCLEAN (VALUE)

c  Translate value from ASCII to binary

	If (FOUND) Then
	    Call VLEN (VALUE,LVAL)
	    Call ASCBINI2 (LVAL,VALUE,IVAL,IER)
	    Endif
	Return
	End


c--- HGETR4  Extract Real*4 value for variable from FITS header string

	Subroutine HGETR4 (HSTRING,VNAME,RVAL)

	Character*(*) HSTRING
c		Character string containing FITS-style header information
	Character*(*) VNAME
c		Character string containing the name of the variable
	Real*4 RVAL

	Character*30 VALUE
	Integer*4 LVAL,IER
	Include 'header.cmb'

c  Get value and comment from header string

	Call HGETC (HSTRING,VNAME,VALUE)

c  Translate value from ASCII to binary

	If (FOUND) Then
	    Call VLEN (VALUE,LVAL)
	    Call ASCBINR4 (LVAL,VALUE,RVAL,IER)
	    Endif
	Return
	End


c--- HGETRA  Extract Real*8 right ascension in radians from FITS header string

	Subroutine HGETRA (HSTRING,VNAME,RA)

	Character*(*) HSTRING
c		Character string containing FITS-style header information
	Character*(*) VNAME
c		Character string containing the name of the variable
	Real*8 RA

	Real*8 HRA

	HRA = -99.d0
	Call HGETR8 (HSTRING,VNAME,HRA)

	If (HRA .gt. -99.d0) Then
	    Call RCONR (HRA,RA)
	    Endif
	Return
	End


c--- HGETDEC  Extract Real*8 declination in radians from FITS header string

	Subroutine HGETDEC (HSTRING,VNAME,DEC)

	Character*(*) HSTRING
c		Character string containing FITS-style header information
	Character*(*) VNAME
c		Character string containing the name of the variable
	Real*8 DEC

	Real*8 HDEC

	HDEC = -99.d0
	Call HGETR8 (HSTRING,VNAME,HDEC)

	If (HDEC .gt. -99.d0) Then
	    Call DCONR (HDEC,DEC)
	    Endif
	Return
	End


c--- HGETANG  Extract Real*8 angle in radians from FITS header string

	Subroutine HGETANG (HSTRING,VNAME,ANG)

	Character*(*) HSTRING
c		Character string containing FITS-style header information
	Character*(*) VNAME
c		Character string containing the name of the variable
	Real*8 ANG

	Character*30 VALUE
	Integer*4 LVAL,IER
	Real*8 DVAL
	Include 'header.cmb'
	Include 'constants.cmb'

c  Get value and comment from header string

	Call HGETC (HSTRING,VNAME,VALUE)

	If (.not. FOUND) Return

	Call VLEN (VALUE,LVAL)
	Call ASCBINR8 (LVAL,VALUE,DVAL,IER)
	If (Index (VALUE,':') .gt. 0) Then
	    Call DCONR (DVAL, ANG)
	Else
	    Call CONSET
	    ANG = DVAL * RCON
	Endif

	Return
	End


c--- HGETR8  Extract Real*8 value for variable from FITS header string

	Subroutine HGETR8 (HSTRING,VNAME,DVAL)

	Character*(*) HSTRING
c		Character string containing FITS-style header information
	Character*(*) VNAME
c		Character string containing the name of the variable
	Real*8 DVAL

	Character*30 VALUE
	Integer*4 LVAL,IER
	Include 'header.cmb'

c  Get value and comment from header string

	Call HGETC (HSTRING,VNAME,VALUE)

c  Translate value from ASCII to binary

	If (FOUND) Then
	    Call VLEN (VALUE,LVAL)
	    Call ASCBINR8 (LVAL,VALUE,DVAL,IER)
	    Endif
	Return
	End


c--- HGETL  Extract Logical value for variable from FITS header string

	Subroutine HGETL (HSTRING,VNAME,LVAL)

	Character*(*) HSTRING
c		Character string containing FITS-style header information
	Character*(*) VNAME
c		Character string containing the name of the variable
	Logical LVAL

	Character*30 VALUE
	Integer*4 IER
	Include 'header.cmb'

c  Get value and comment from header string

	Call HGETC (HSTRING,VNAME,VALUE)

c  Translate value from ASCII to binary

	If (FOUND) Then
	    Call ASCBINL (VALUE,LVAL,IER)
	    Endif
	Return
	End


c HGETJD -- Read Julian Date from FITS header or make it up if not there

	Subroutine HGETJD (HEADER, DJ)

	Character*(*) HEADER
c				FITS header
	Real*8 DJ
c				Julian date (returned)
c				(0 if not found)

	Real*8 UT, DATE, TIME, EXP
	Integer*4 YR, MN, DAY
	DJ = 0.d0

c  Try a couple of keywords
	Call HGETR8 (HEADER,'JULDATE',DJ)
	If (DJ .le. 0.d0) Call HGETR8 (HEADER,'GJDN',DJ)
	If (DJ .le. 0.d0) Call HGETR8 (HEADER,'JDN',DJ)

c  If they're not there, use the observation date and UT
	If (DJ .le. 0.d0) Then
	    UT = -1.d0
	    YR = -1
	    Call HGETDATE (HEADER,'DATE-OBS',YR,MN,DAY,UT)
	    If (YR .gt. 0) Then
		DATE = Dble(YR) + 1.d-2 * Dble(MN) + 1.d-4 * Dble(DAY)
		If (UT .ge. 0.d0) Then
		    Call HCON2 (UT, TIME)
		Else
		    EXP = 0.d0
		    Call HGETR8 (HEADER,'EXPTIME', EXP)
		    If (EXP .eq. 0.d0) Then
			Call HGETR8 (HEADER,'EXPOSURE', EXP)
			Endif
		    TIME = -1.d0
		    Call HGETR8 (HEADER,'UTMID',TIME)
		    If (TIME .lt. 0.d0) Then
			Call HGETR8 (HEADER,'UTOPEN',TIME)
			If (TIME .ge. 0.d0) Then
			    Call HCON (TIME, UT)
			    TIME = TIME + (0.5d0 * EXP / 3600.d0)
			    Call HCON2 (UT, TIME)
			Else
			    Call HGETR8 (HEADER,'UTCLOSE',TIME)
			    If (TIME .lt. 0.0d0) Then
				Call HGETR8 (HEADER,'UTEND',TIME)
				Endif
			    If (TIME .ge. 0.d0) Then
				Call HCON (TIME, UT)
				UT = UT - (0.5d0 * EXP / 3600.d0)
				Call HCON2 (UT, TIME)
				Endif
			Endif
			Endif
		Endif
		Call JCON (DATE, TIME, DJ)
		Endif
	    Endif
	Return
	End


c--- HGETDATE returns date and time

	Subroutine HGETDATE (HSTRING,VNAME,YR,MN,DAY,UT)

	Character*(*) HSTRING
c		Character string containing FITS-style header information
	Character*(*) VNAME
c		Character string containing the name of the variable
	Integer*4 YR, MN, DAY
c		Date as 4-digit year, month, and day (Returned)
	Real*8 UT
c		UT (or time, anyway) as fractional hours (Returned)
c		(If not set, -1 returned)

	Character*1 SLASH, DASH, COLON
	Integer*4 IX1, IX2, IX3, IX4, IX5, IX6, NC, LV
	Real*8 HR, MIN, SEC
	Character*32 VALUE
	Integer*4 IER
	Include 'header.cmb'

	YR = 0
	MN = 0
	DAY = 0
	UT = -1.d0
	SLASH = '/'
	DASH = '-'
	COLON = ':'

c  Get value and comment from header string
	Call HGETC (HSTRING,VNAME,VALUE)
	If (.not. FOUND) Return

c  Check for old FITS date format
	IX1 = Index (VALUE,SLASH)
	IF (IX1 .gt. 0) Then

c  Extract date from old FITS date format (allowing optional 4-digit year)
	    NC = IX1 - 1
	    Call ASCBINI4 (NC,VALUE,DAY,IER)
	    IX2 = IX1 + Index (VALUE(IX1+1:),SLASH)
	    NC = IX2 - IX1 - 1
	    Call ASCBINI4 (NC,VALUE(IX1+1:),MN,IER)
	    Call VLEN (VALUE, LV)
	    NC = LV - IX2
	    Call ASCBINI4 (NC,VALUE(IX2+1:),YR,IER)
	    If (YR .lt. 30) Then
		YR = YR + 2000
	    Elseif (YR .lt. 150) Then
		YR = YR + 1900
	    Endif
	    Return
	    Endif

c  Check for ISO date format ('ccyy-mm-dd[Thh:mm:ss[.sss...]]'
	IX1 = Index (VALUE, DASH)
	IF (IX1 .gt. 0) Then

c  Extract date from ISO format
	    NC = IX1 - 1
	    Call ASCBINI4 (NC,VALUE,YR,IER)
	    IX2 = IX1 + Index (VALUE(IX1+1:),DASH)
	    NC = IX2 - IX1 - 1
	    Call ASCBINI4 (NC,VALUE(IX1+1:),MN,IER)
	    Call VLEN (VALUE, LV)
	    IX3 = Index (VALUE, 'T')
	    If (IX3 .gt. 0) Then
		NC = IX3 - IX2
	    Else
		NC = LV - IX2 + 1
	    Endif
	    Call ASCBINI4 (NC,VALUE(IX2+1:),DAY,IER)
	    If (IX3 .le. 0) Return

c  Extract time from ISO format, if it is present
	    HR = 0.d0
	    MIN = 0.d0
	    SEC = 0.d0
	    IX4 = Index (VALUE,COLON)
	    NC = IX4 - IX3 - 1
	    Call ASCBINR8 (NC,VALUE(IX3+1:),HR,IER)
	    IX5 = IX4 + Index (VALUE(IX4+1:),COLON)
	    If (IX5 .gt. IX4) Then
		NC = IX5 - IX4 - 1
		Call ASCBINR8 (NC,VALUE(IX4+1:),HR,IER)
	    	IX6 = Index (VALUE,'Z')
		If (IX6 .gt. 0) Then
		    NC = IX6 - IX5
		Else
		    NC = LV - IX5
		Endif
		If (NC .gt. 0) Then
		    Call ASCBINR8 (NC,VALUE(IX5+1:),SEC,IER)
		    Endif
		Endif
	    UT = HR + (MIN / 60.d0) + (SEC / 3.6d3)
	    Endif

	Return
	End

c--- HGETC  Extract Character value for variable from FITS header string

	Subroutine HGETC (HSTRING,VNAME,CVAL)

	Character*(*) HSTRING
c		Character string containing FITS-style header information
	Character*(*) VNAME
c		Character string containing the name of the variable
	Character*(*) CVAL
c			Character string containing the value for variable
c			VNAME.  Trailing and leading blanks are removed.

	Character*1 NULL,SQUOT,DQUOT
	Character*100 LINE
	Character*80 TEMP
	Integer*4 LVNAME,LVAL
	Integer*4 L0,L1,LQ0,LQ1,LV,LV0,LV1,LC,LC0,LC1,LLIN
	Integer*4 IBRACK1,IBRACK2,NC,IPAR,IER
	Integer*4 NTOK,NTMAX,TOK1(20),TOK2(20)

	Include 'header.cmb'

	Common/HGCOM/ PCOM
	Character*80 PCOM

	SQUOT = Char (39)
	DQUOT = Char (34)
	NULL = Char (0)
	NTMAX = 20

	PCOM(1:1) = NULL
	FOUND = .false.

c  Find length of variable name

	Call VLEN (VNAME,LVNAME)
	IBRACK1 = Index (VNAME,'[')
	If (IBRACK1 .le. 0) IBRACK1 = Index (VNAME,',')
	If (IBRACK1 .gt. 0) LVNAME = IBRACK1 - 1

c  Search header string for variable name

	Call HSEARCH (HSTRING,VNAME(1:LVNAME),L0)

c  Exit if not found

	If (L0 .le. 0) Then
	    If (VERBOSE) Then
		Write(LINE,31) VNAME(1:LVNAME)
31		Format('HGETC: "',A,'" not found$')
		Call PSTRING (LINE)
		Endif
	    Return
	    Endif

c  Set flag indicating that the variable was found in the header

	FOUND = .true.

c  In standard FITS, data lasts until 80th character

	L1 = L0 + 79

c  Extract entry for this variable from the header

	LINE = HSTRING(L0:L1)
	LLIN = 80

c  Check for quoted value

	LQ0 = Index (LINE,SQUOT)
	If (LQ0 .gt. 0) Then
	    LQ1 = LQ0 + Index (LINE(LQ0+1:LLIN),SQUOT)
	Else
	    LQ0 = Index (LINE,DQUOT)
	    If (LQ0 .gt. 0) Then
		LQ1 = LQ0 + Index (LINE(LQ0+1:LLIN),DQUOT)
	    Else
		LQ0 = 0
		LQ1 = 10
	    Endif
	Endif

c  Extract value and remove excess spaces

	LC0 = Index (LINE(LQ1+1:LLIN),'/')
	If (LQ0 .gt. 0) Then
	    LV0 = LQ0 + 1
	    LV1 = LQ1 - 1
	Else
	    LV0 = Index (LINE,'=') + 1
	    If (LV0 .le. 1) Then
		LV0 = Index (LINE,':') + 1
		Endif
	    If (LV0 .le. 1) Then
		LV0 = Index (LINE,' ') + 1
		Endif
	    If (LC0 .gt. 0) Then
		LV1 = LQ1 + LC0 - 1
	    Else
		LV1 = LLIN
	    Endif
	Endif
	Do 100 LV = LV0, LV1
100	    If (LINE(LV:LV) .ne. ' ') Go to 110
110	LV0 = LV
	Do 120 LV = LV1, LV0, -1
120	    If (LINE(LV:LV) .ne. ' ') Go to 130
130	LV1 = LV
	CVAL = LINE(LV0:LV1)
	Call PADNULL (CVAL)
	LVAL = Len (CVAL)
	If ((LVAL .gt. 0) .and. (VERBOSE)) Then
	    Write(*,140) VNAME(1:LVNAME),CVAL(1:LVAL)
140	    Format(A,' = ',A)
	    Endif

c  Extract comment and remove excess spaces

	If (LC0 .gt. 0) Then
	    LC0 = LQ1 + LC0 + 1
	    LC1 = LLIN
	    Do 200 LC = LC0, LC1
200		If (LINE(LC:LC) .ne. ' ') Go to 210
210	    LC0 = LC
	    Do 220 LC = LC1, LC0, -1
220		If (LINE(LC:LC) .ne. ' ') Go to 230
230	    LC1 = LC
	    PCOM = LINE(LC0:LC1)
	Else
	    PCOM = NULL
	Endif

c  If keyword has brackets, extract appropriate token from value

	If (IBRACK1 .gt. 0) Then
	    IBRACK2 = Index (VNAME,']') - 1
	    If (IBRACK2 .le. 0) Call VLEN (VNAME,IBRACK2)
	    If (IBRACK2 .gt. IBRACK1) Then
		NC = IBRACK2 - IBRACK1
		Call ASCBINI4 (NC,VNAME(IBRACK1+1:IBRACK2),IPAR,IER)
		If (IER .eq. 0) Then
		    Call SETOKEN (CVAL,NTMAX,NULL,NTOK,TOK1,TOK2)
		    Call GETOKEN (CVAL,NTOK,TOK1,TOK2,IPAR,TEMP,LVAL)
		    CVAL = TEMP(1:LVAL)
		    CVAL(LVAL+1:LVAL+1) = NULL
		    Call PADNULL (CVAL)
		    Endif
		Endif
	    Endif

	Return
	End


c--- HGETCOM  Return comment for last parameter that was read

	Subroutine HGETCOM (COMMENT)

	Character*(*) COMMENT
c		Character string containing comment from header

	Common/HGCOM/ PCOM
	Character*80 PCOM

	COMMENT = PCOM
	Call PADNULL (COMMENT)

	Return
	End

c--- HCLEAN  Return integer string free of non-numeric characters

	Subroutine HCLEAN (STRING)

	Character*(*) STRING

	Integer*4 I, J, LSTR, IDOT, IC

	Call VLEN (STRING,LSTR)

c  Round to integer if decimal point is present

	IDOT = Index (STRING,'.')
	If (IDOT .gt. 0) Then
	    Call RNDASC (STRING,0)
	    Do I = IDOT, LSTR
		STRING(I:I) = Char (0)
		Enddo
	    LSTR = IDOT - 1
	    Endif

c  Keep only numeric characters

	J = 0
	Do I = 1, LSTR
	    IC = Ichar (STRING(I:I))
	    If ((I .eq. 1) .and. ((IC .eq. 43) .or. (IC .eq. 45))) Then
		J = J + 1
		STRING(J:J) = STRING(I:I)
	    Elseif ((IC .gt. 47) .and. (IC .lt. 58)) Then
		J = J + 1
		STRING(J:J) = STRING(I:I)
	    Endif
	    Enddo
	If (J .lt. LSTR) Then
	    Do I = J+1, LSTR
		STRING(I:I) = Char (0)
		Enddo
	    Endif
	Return
	End
c Jan 20 1988	Drop COMMENT from argument list, add HGETCOM
c Jan 22 1988	Replace PADSTR call with PADNULL
c Mar  4 1988	Fix HGETC for quoted parameters
c Apr  5 1988	Fix value extraction in HGETC

c Feb 25 1993	Change double quotes to single quotes

c Jul  5 1994	Add option to specify one token of multiplexed keyword
c Dec  5 1994	Allow values after colon if no equal sign present
c Dec  6 1994	Fix character value parsing code; allow , delimiter
c Dec  6 1994	For integer output, remove non-numeric characters and round
c Dec  6 1994	Use typed calles to ascbin

c Jan 20 1995	Drop length from ascbinl call
c Jan 23 1995	Keep sign if cleaning integer
c Jan 25 1995	Declare undeclared variable

c Dec 31 1996	Add hgetra and hgetdec to get right ascension and declination

c Jan  3 1997	Add hgetdate to get date in either old or new format
c Apr 16 1997	Declare undeclared variables in hgetdate
c Jun  2 1997	Return UT as -1 from HGETDATE if not set
c Jun  3 1997	Add hgetjd to return Julian Date, computing it from UT date

c Jan  3 2000	If old FITS year is 30 - 150, add 1900, < 30, add 2000
c Jan  3 2000	For HGETJD(), compute midtime from UTMID, UTOPEN, UTCLOSE

c Nov  8 2001	Declare undeclared variables

c Jun 18 2003	Clean up HGETJD, add JDN possibility
c Sep 18 2003	Declare EXP; call ascbin() in hget()
