c*** File fitslib/irafrhead.f
c*** August 20, 1998
c*** By Doug Mink, Harvard-Smithsonian Center for Astrophysics

	Subroutine IRAFRHEAD (INPATH,HEADER,NBHEAD)

	Character*(*) INPATH
c				Input image header pathname
	Character*(*) HEADER
c				Image FITS header (returned)
	Integer*4 NBHEAD
c				Length of FITS header (returned)

	Integer*4 IFD
c				Unix file descriptor
	Character*79 OBJNAME
c				Object name from FITS file
	Integer*4 IMHVER
c				1 = IRAF 2.10 and earlier
c				2 = IRAF 2.11 and later
	Integer*4 IER,IBPX
	Integer*4 I, J, K, ITEMP
	Integer*2 HTEMP
	Logical SWAP
	Character*1 IRAFCHAR
	Character*82 FITSLINE
	Character*50000 IRAFHEADER
	Integer*4 IMNDIM, IMPHYSLEN, IMPIXTYPE, IMPIXOFF, IMTITLE, PIXTYPE
	Integer*4 LIN, NBYTES, LFHEAD, NAX, IMU

c  Open input file
	IFD = -1
	Call OPENR (IFD,INPATH)
	If (IFD .lt. 3) Then
	    Call VLEN (INPATH,LIN)
	    Write(*,50) INPATH(1:LIN)
50	    Format('*** File "',A,'" cannot be read')
	    NBHEAD = 0
	    Return
	    Endif

c  Read IRAF header
	NBYTES = 50000
	Call SGETC (IFD,NBYTES,IRAFHEADER,NBHEAD)
	Call SCLOSE (IFD,IER)

c  Initialize FITS header
	HEADER = 'END$'
	Call PADSPACE (HEADER)
	Call HPUTL (HEADER, 'SIMPLE', .true.)

c  Get IRAF image format version and character-swapping for version 1
	Call HDRVERSION (IRAFHEADER, IMHVER, SWAP)
	If (IMHVER .eq. 2) Then
	    IMNDIM = 18
	    IMPHYSLEN = 50
	    IMPIXTYPE = 10
	    IMPIXOFF = 86
	    IMTITLE = 638
	Else
	    IMNDIM = 20
	    IMPHYSLEN = 52
	    IMPIXTYPE = 16
	    IMPIXOFF = 88
	    IMTITLE = 732
	Endif

c  Set up pixel size in FITS header
	Call STGETI4 (IRAFHEADER, IMPIXTYPE, PIXTYPE)
	If (SWAP) Call SSWAP4 (PIXTYPE, 4)
	If (PIXTYPE .eq. 2) Then
	    IBPX = 8
	Elseif (PIXTYPE .eq. 3) Then
	    IBPX = 16
	Elseif (PIXTYPE .eq. 4) Then
	    IBPX = 32
	Elseif (PIXTYPE .eq. 5) Then
	    IBPX = 32
	Elseif (PIXTYPE .eq. 6) Then
	    IBPX = -32
	Elseif (PIXTYPE .eq. 7) Then
	    IBPX = -64
	Elseif (PIXTYPE .eq. 11) Then
	    IBPX = -16
	Elseif (PIXTYPE .eq. 12) Then
	    IBPX = -8
	Else
	    Write(*,*) 'IRAFRHEAD: Illegal data type ',PIXTYPE
	    IBPX = 16
	Endif
	Call HPUTI4 (HEADER,'BITPIX',IBPX)
	Call HPUTCOM (HEADER, 'BITPIX', 'IRAF .imh pixel type')

c  Set up image dimensions in FITS header
	Call STGETI4 (IRAFHEADER, IMNDIM, NAX)
	If (SWAP) Call SSWAP4 (NAX, 4)
	Call HPUTI4 (HEADER,'NAXIS',NAX)
	Call HPUTCOM (HEADER, 'NAXIS', 'IRAF .imh naxis')

	Call STGETI4 (IRAFHEADER, IMPHYSLEN, ITEMP)
	If (SWAP) Call SSWAP4 (ITEMP, 4)
	Call HPUTI4 (HEADER,'NAXIS1',ITEMP)
	LFHEAD = 320
	If (NAX .gt. 1) Then
	    Call STGETI4 (IRAFHEADER, IMPHYSLEN+4, ITEMP)
	    If (SWAP) Call SSWAP4 (ITEMP, 4)
	    Call HPUTI4 (HEADER,'NAXIS2',ITEMP)
	    Call  HPUTCOM (HEADER,'NAXIS2', 'IRAF .imh naxis[2]')
	    LFHEAD = LFHEAD + 80
	    Endif
	If (NAX .gt. 2) Then
	    Call STGETI4 (IRAFHEADER, IMPHYSLEN+8, ITEMP)
	    If (SWAP) Call SSWAP4 (ITEMP, 4)
	    Call HPUTI4 (HEADER,'NAXIS3',ITEMP)
	    Call  HPUTCOM (HEADER,'NAXIS3', 'IRAF .imh naxis[3]')
	    LFHEAD = LFHEAD + 80
	    Endif
	If (NAX .gt. 3) Then
	    Call STGETI4 (IRAFHEADER, IMPHYSLEN+12, ITEMP)
	    If (SWAP) Call SSWAP4 (ITEMP, 4)
	    Call HPUTI4 (HEADER,'NAXIS4',ITEMP)
	    Call  HPUTCOM (HEADER,'NAXIS4', 'IRAF .imh naxis[4]')
	    LFHEAD = LFHEAD + 80
	    Endif

c  Set object name in FITS header
	OBJNAME = ' '
	Call PADSPACE (OBJNAME)
	If (IMHVER .eq. 2) Then
	    Call STGETC (IRAFHEADER, IMTITLE, 32, OBJNAME)
	Else
	    Do I = 1, 32
		Call STGETI2 (IRAFHEADER, IMTITLE+2*(I-1), HTEMP)
		If (SWAP) Call SSWAP2 (HTEMP, 2)
		OBJNAME(I:I) = Char (HTEMP)
		Enddo
	Endif
	OBJNAME(32:32) = Char (0)
	Call HPUTS (HEADER,'OBJECT',OBJNAME)
	Call HPUTCOM (HEADER,'OBJECT','IRAF .imh title')
	LFHEAD = LFHEAD + 80

c  Save image header filename in header */
	Call HPUTS (HEADER, 'IMHFILE', INPATH)
	LFHEAD = LFHEAD + 80

c  Save IRAF file format version in header */
	Call HPUTI4 (HEADER, 'IMHVER', IMHVER)
	Call HPUTCOM (HEADER, 'IMHVER', 'IRAF .imh format version (1 or 2)')
	LFHEAD = LFHEAD + 80

c  Add "user" portion of IRAF header to FITS HEADER
	FITSLINE(81:81) = Char (0)

c  Read version 2 .imh header
	If (IMHVER .eq. 2) Then
	    IMU = 2047
	    J = 1
	    Do K = 1, 80
		FITSLINE(k:k) = ' '
		Enddo
	    Do I = IMU, NBHEAD
		IRAFCHAR = IRAFHEADER(I:I)
		If (IRAFCHAR .eq. Char(0)) Then 
		    Go to 100
		Elseif (IRAFCHAR .eq. Char (10)) Then
		    Do K = 1, 80
			HEADER(LFHEAD+K:LFHEAD+K) = FITSLINE(K:K)
			Enddo
		    J = 1
		    LFHEAD = LFHEAD + 80
		    Do K = 1, 80
			FITSLINE(K:K) = ' '
			Enddo
		Else
		    If (J .gt. 80) Then
			Do K = 1, 80
			    HEADER(LFHEAD+K:LFHEAD+K) = FITSLINE(K:K)
			    Enddo
			J = 10
			LFHEAD = LFHEAD + 80
			Do K = 1, 80
			    FITSLINE(K:K) = ' '
			    Enddo
			Endif
		    If ((IRAFCHAR.gt.' ') .and. (IRAFCHAR.lt.Char(127))) Then
			FITSLINE(J:J) = IRAFCHAR
			Endif
		    J = J + 1
		Endif
		Enddo

c  Read version 1 .imh header
	Else
	    IMU = 2052
	    Do K = 1, 80
		FITSLINE(k:k) = ' '
		Enddo
	    J = 1
	    Do I = IMU, NBHEAD, 2
		Call STGETI2 (IRAFHEADER, I, HTEMP)
		If (SWAP) Call SSWAP2 (HTEMP, 2)
		IRAFCHAR = Char (HTEMP)
		If (IRAFCHAR .eq. Char(0)) Then 
		    Go to 100
		Elseif (IRAFCHAR .eq. Char (10)) Then
		    Do K = 1, 80
			HEADER(LFHEAD+K:LFHEAD+K) = FITSLINE(K:K)
			Enddo
		    J = 1
		    LFHEAD = LFHEAD + 80
		    Do K = 1, 80
			FITSLINE(K:K) = ' '
			Enddo
		Else
		    If (J .gt. 80) Then
			Do K = 1, 80
			    HEADER(LFHEAD+K:LFHEAD+K) = FITSLINE(K:K)
			    Enddo
			J = 10
			LFHEAD = LFHEAD + 80
			Do K = 1, 80
			    FITSLINE(K:K) = ' '
			    Enddo
			Endif
		    If ((IRAFCHAR.gt.' ') .and. (IRAFCHAR.lt.Char (127))) Then
			FITSLINE(J:J) = IRAFCHAR
			Endif
		    J = J + 1
		Endif
		Enddo
	Endif

c  Add END to last line
100	LFHEAD = (LFHEAD / 80) * 80
	HEADER(LFHEAD+1:LFHEAD+4) = 'END'
	HEADER(LFHEAD+5:LFHEAD+5) = Char (0)

c  Pad rest of header with spaces
	Call PADSPACE (HEADER)
	NBHEAD = LFHEAD + 80

	Return
	End

	Subroutine HDRVERSION (IRAFHEADER, VERSION, SWAP)

	Character*(*) IRAFHEADER
c					Entire IRAF header
	Integer*4 VERSION
c					.IMH format version (1 or 2 returned)
	Logical SWAP
c					True if bytes swapped from this CPU

	Integer*2 IMTYPE(5)
	Character*10 CIMTYPE, IMHCODE
	Equivalence (IMTYPE, CIMTYPE)
	Integer*4 I

	CIMTYPE(1:10) = IRAFHEADER(1:10)
	VERSION = 0

	If (IRAFHEADER(1:5) .eq. 'imhv2') Then
	    VERSION = 2
	Else
	    Do I = 1, 5
		IMHCODE(I:I) = Char (IMTYPE(I))
		Enddo
	    If (IMHCODE(1:5) .eq. 'imhdr') Then
		VERSION = 1
		SWAP = .false.
	    Else
		Call SSWAP2 (IMTYPE, 10)
		Do I = 1, 5
		    IMHCODE(I:I) = Char (IMTYPE(I))
		    Enddo
		If (IMHCODE(1:5) .eq. 'imhdr') Then
		    VERSION = 1
		    SWAP = .true.
		    Endif
	    Endif
	Endif

	Return
	End

c Jul  1 1994	New program

c Nov 20 1994	Fix user area reading

c Jan  5 1995	Fix dimension error concerning objname

c May 13 1996	Increase IRAF header buffer from 20000 to 30000

c Jul 23 1998	Add capability of reading IMH version 2 (IRAF 2.11+)
c Jul 23 1998	Increase maximum header file size from 30000 to 50000 bytes
c Aug 20 1998	Declare undeclared variables
