/*** File eset.c
 *** November 16, 2001
 *** By Chip Coldwell and Doug Mink
 */

/*   Fortran-callable program to set terminal to non-raw, echoing mode
     if terminal is not already set
 */

#define TERMIOS
#ifdef TERMIOS
#include <termios.h>
#include <unistd.h>
#else
#include <sgtty.h>
#endif

int
eset_ ()


#ifdef TERMIOS
{
  struct termios tios;
  int retval;
  void cfmakesane();

  if( (retval = tcgetattr(STDIN_FILENO, &tios)) < 0)
    return retval;
  cfmakesane(&tios);
  return tcsetattr(STDIN_FILENO, TCSAFLUSH, &tios);
}

/* I don't know why this function isn't in the standard C library when
   they do provide a "cfmakeraw".  For a justification of the settings
   below, see the manpage for stty, especially the description of the
   action of the command "stty sane". */

void
cfmakesane (struct termios *tios)
{
  tios->c_iflag |= (BRKINT | ICRNL | IMAXBEL);
  tios->c_iflag &= ~(IGNBRK | INLCR | IGNCR | IUCLC | IXANY | IXOFF);

  tios->c_oflag |= (OPOST | ONLCR | NL0 | CR0 | TAB0 | BS0 | VT0 | FF0);
  tios->c_oflag &= ~(OLCUC | OCRNL | ONOCR | ONLRET | OFILL | OFDEL);

  tios->c_cflag |= (CREAD);

  tios->c_lflag |= (ISIG | ICANON | ECHO | ECHOE | ECHOK | ECHOCTL |
                   ECHOKE | IEXTEN);
  tios->c_lflag &= ~(XCASE | ECHONL | ECHOPRT | NOFLSH | TOSTOP);
}  
#else
{
struct sgttyb arg;
int fildes,code,ier;
	fildes = 0;
	ier = ioctl (fildes,TIOCGETP,&arg);
	if ((arg.sg_flags & RAW) || !(arg.sg_flags & ECHO)) { 
	   arg.sg_flags = arg.sg_flags & ~RAW;
	   arg.sg_flags = arg.sg_flags | ECHO;
	   ier = ioctl (fildes,TIOCSETP,&arg); }
	return;
}
#endif

/* Jun  4 1984	New subroutine
 *
 * Nov 13 2001	Modified by Chip Coldwell to use TERMIO library
 * Nov 16 2001	Set TERMIOS here
 */
