c*** File iolib/hcon.f
c*** December 1, 2004
c*** By Doug Mink

c  Angle conversion subroutines

c	HCON (DEG,FDEG) dd.mmssss to fractional degrees.
c	HCON1 (FDEG,ISGN,ID,IM,DSEC) fractional degrees to deg, min, sec
c	HCON2 (FDEG,DEG) fractional degrees to degrees as dd.mmssssss.
c	HCON3 (FDEG,ISGN,ID,IM,ISEC) fractional degrees to deg, min, int. sec
c	HCONR (DEG,FRAD) dd.mmssss to radians.
c	HCONS (FRAD,DEG) radians to dd.mmssss
c	HCSTRH (DEC,NDEC,STRING) radians to string (dd.mmssss); drop zeroes
c	HCSTRT (DEC,NDEC,STRING) radians to string (d'"); drop zeroes
c	HCSTRC (DEC,NDEC,STRING) radians to string (:); drop zeroes
c	HCSTRS (DEC,NDEC,STRING) radians to seconds; drop zeroes

c--- HCON  Convert from degrees as dd.mmssss to fractional degrees.

	Subroutine HCON (DEG,FDEG)

	Real*8 DEG
c			Angle in degrees (as dd.mmsssss)
	Real*8 FDEG
c			Angle in degrees (as fractional degrees)

	Logical NEGATIVE
	Real*8 XNUM,XD,XM,XS

	XNUM = DEG
	NEGATIVE = .false.
	If (XNUM .lt. 0.d0) Then
	    NEGATIVE = .true.
	    XNUM = -XNUM
	    Endif

	XD = Dble (Idint (XNUM + 1.d-9))
	XNUM = (XNUM - XD) * 1.d2

	XM = Dble (Idint (XNUM + 1.d-9))
	XS = (XNUM - XM) * 1.d2

	FDEG = (XD + (XM / 6.d1) + (XS / 3.6d3))
	If (NEGATIVE) FDEG = -FDEG

	Return
	End


c--- HCON1  Convert fractional degrees to degrees, minutes, and seconds.

	Subroutine HCON1 (FDEG,ISGN,ID,IM,DSEC)

	Real*8 FDEG
c			Fractional degrees or hours
	Integer*4 ISGN
c			Sign (1 or -1) (returned)
	Integer*4 ID
c			Degrees or hours (returned)
	Integer*4 IM
c			Minutes (returned)
	Real*8 DSEC
c			Seconds (returned)

	Real*8 DEG,DSGN,DM,C36,C60,C59
	Integer*4 NDEG,NMIN

	C59 = 59.999d0
	C60 = 6.d1
	C36 = 3.6d3
	DEG = FDEG
	DSGN = 1.d0
	If (DEG .lt. -0.0001) Then
	    DSGN = -1.d0
	    DEG = -DEG
	    Endif
	DEG = DEG * C36

	NMIN = Idint (DEG / C60)
	DSEC = DEG - (C60 * Dble (NMIN))
c	DSEC = Dmod (DEG, C60)
	If (DSEC .lt. 0.d0) DSEC = 0.d0
	If (DSEC .gt. C59) DSEC = 0.d0
	DEG = (DEG - DSEC) / C60
	NDEG = Idint (DEG / C60)
	DM = DEG - (C60 * Dble (NDEG))
c	DM = Dmod (DEG, C60)
	If (DM .gt. C59) DM = 0.d0
	IM = Idnint (DM)
	DEG = (DEG - Dble (IM)) / 60.d0
	ID = Idnint (DEG)
	ISGN = Idnint (DSGN)

	Return
	End


c--- HCON2  Convert fractional degrees to degrees as dd.mmssssss.

	Subroutine HCON2 (FDEG,DEG)

	Real*8 FDEG
c			Angle in degrees (as fractional degrees)
	Real*8 DEG
c			Angle in degrees (as dd.mmsssss)

	Real*8 DSGN,DSEC,DM
	Integer*4 IM,ID

	DEG = FDEG * 3.6d3
	DSGN = 1.d0
	If (DEG .lt. 0) Then
	    DSGN = -1.d0
	    DEG = -DEG
	    Endif
	DSEC = Dmod (DEG, 60.d0)
	If (DSEC .gt. 59.9999) DSEC = 0.d0
	If (DSEC .lt. 0.d0) DSEC = 0.d0
	DEG = (DEG - DSEC) / 60.d0
	DM = Dmod (DEG, 60.d0)
	If (DM .gt. 59.9999) DM = 0.d0
	IM = Idnint (DM)
	DEG = (DEG - Dble (IM)) / 60.d0
	ID = Idnint (DEG)
	DEG = DSGN * (Dble(ID) + (1.d-2 * Dble(IM)) + (1.d-4 * DSEC))

	Return
	End


c--- HCON3  Convert fractional degrees to degrees, minutes, and integer seconds.

	Subroutine HCON3 (FDEG,ISGN,ID,IM,ISEC)

	Real*8 FDEG
c			Fractional degrees or hours
	Integer*4 ISGN
c			Sign (1 or -1) (returned)
	Integer*4 ID
c			Degrees or hours (returned)
	Integer*4 IM
c			Minutes (returned)
	Integer*4 ISEC
c			Seconds (returned)

	ISGN = 1
	ID = 0
	IM = 0
	ISEC =Idnint (FDEG * 3.6d3)
	If (ISEC .eq. 0) Return
	If (ISEC .lt. 0) Then
	    ISGN = -1
	    ISEC = -ISEC
	    Endif
	ID = ISEC / 3600
	ISEC = ISEC - (ID * 3600)
	IM = ISEC / 60
	ISEC = ISEC - (IM * 60)

	Return
	End


c--- HCONR  Convert from degrees as dd.mmssss to radians.

	Subroutine HCONR (DEG,FRAD)

	Real*8 DEG
c			Angle in degrees (as dd.mmsssss)
	Real*8 FRAD
c			Angle in radians (returned)

	Real*8 XNUM,SGN,XD,XM,XS
	Integer*4 IDEG,IMIN

	XNUM = DEG
	SGN = 1.d0
	If (XNUM .lt. 0.d0) Then
	    SGN = -1.d0
	    XNUM = -XNUM
	    Endif

	IDEG = Idint (XNUM + 1.d-9)
	XD = Dble (IDEG)
	XNUM = (XNUM - XD) * 1.d2
	IMIN = Idint (XNUM + 1.d-9)
	XM = Dble (IMIN)
	XS = (XNUM - XM) * 1.d2

	FRAD = SGN * (XD + (XM / 6.D1) + (XS / 3.6D3)) * 1.74532925199433d-2

	Return
	End


c--- HCONS  Convert radians to degrees as dd.mmssssss.

	Subroutine HCONS (FRAD,DEG)

	Real*8 FRAD
c			Angle in radians
	Real*8 DEG
c			Angle in degrees (as dd.mmsssss)

	Real*8 FDEG, DSGN,DSEC,DM
	Integer*4 IM,ID

	FDEG = FRAD / 1.74532925199433d-2
	DEG = FDEG * 3.6d3
	DSGN = 1.d0
	If (DEG .lt. 0) Then
	    DSGN = -1.d0
	    DEG = -DEG
	    Endif
	DSEC = Dmod (DEG, 60.d0)
	If (DSEC .gt. 59.9999) DSEC = 0.d0
	If (DSEC .lt. 0.d0) DSEC = 0.d0
	DEG = (DEG - DSEC) / 60.d0
	DM = Dmod (DEG, 60.d0)
	If (DM .gt. 59.9999) DM = 0.d0
	IM = Idnint (DM)
	DEG = (DEG - Dble (IM)) / 60.d0
	ID = Idnint (DEG)
	DEG = DSGN * (Dble(ID) + (1.d-2 * Dble(IM)) + (1.d-4 * DSEC))

	Return
	End


c--- HCSTR  Convert angle in degrees to string

	Subroutine HCSTR (FDEG,NDEC,NPAD,UNITS,STRING)

	Real*8 FDEG
c			Fractional degrees
	Integer*4 NDEC
c			Number of decimal places in seconds
	Integer*4 NPAD
c			Number of spaces between entries
	Logical UNITS
c			If true, print units
	Character*(*) STRING
c			String (returned)

	Integer*4 NFIELD
	Character*10 LFORMAT,TEMP
	Character*1 JDQ,JSQ
	Integer*4 IDD,IDM,ISGN,ISEC,I
	Real*8 DSEC

	JDQ = Char (34)
	JSQ = Char (39)
	If (NDEC .gt. 0) Then
	    Call HCON1 (FDEG,ISGN,IDD,IDM,DSEC)
	Else
	    Call HCON3 (FDEG,ISGN,IDD,IDM,ISEC)
	Endif

c  Degrees

	If (ISGN .lt. 1) Then
	    STRING = '-'
	Else
	    STRING = ' '
	Endif
	Call PADNULL (STRING)
	If (IDD .gt. 99) Then
            Write(TEMP,10) IDD
10	    Format(I3,'$')
	Else
            Write(TEMP,11) IDD
11	    Format(I2,'$')
	Endif
	Call CONSTR (STRING,TEMP)
	If (UNITS) Call CONSTR (STRING,'d')
	If (NPAD .gt. 0) Then
	    Do 15 I = 1, NPAD
15		Call CONSTR (STRING,' $')
	    Endif

c  Minutes

        Write(TEMP,20) IDM
20	Format(I2,'$')
	If (TEMP(1:1) .eq. ' ') TEMP(1:1) = '0'
	Call CONSTR (STRING,TEMP)
	If (UNITS) Call CONSTR (STRING,JSQ)
	If (NPAD .gt. 0) Then
	    Do 25 I = 1, NPAD
25		Call CONSTR (STRING,' $')
	    Endif

c  Seconds

	NFIELD = NDEC + 3
	If (NDEC .le. 0) Then
	    Write(TEMP,'(I2)') ISEC
	Else
	    If (NFIELD .gt. 9) Then
		Write(LFORMAT,35) NFIELD,NDEC
35		Format('(F',I2,'.',I1,')')
	    Else
		Write(LFORMAT,36) NFIELD,NDEC
36		Format('(F',I1,'.',I1,')')
	    Endif
	    Write(TEMP,LFORMAT) DSEC
	    If (TEMP(4:4) .eq. ' ') Then
		Do 40 I = 4, NFIELD
40		    TEMP(I:I) = '0'
		Endif
	Endif
	If (TEMP(1:1) .eq. ' ') TEMP(1:1) = '0'
	Call PADNULL (TEMP)
	Call CONSTR (STRING,TEMP)
	If (UNITS) Call CONSTR (STRING,JDQ)
	If (NPAD .gt. 0) Then
	    Do 45 I = 1, NPAD
45		Call CONSTR (STRING,' $')
	    Endif

	Return
	End


c--- HCSTRT  Convert declination to string and drop trailing zeroes

	Subroutine HCSTRT (FRAD,NDEC,STRING)

	Real*8 FRAD
c			Fractional degrees
	Integer*4 NDEC
c			Number of decimal places in seconds
	Character*(*) STRING
c			String (returned)
	Real*8 FDEG
	Integer*4 LT
	Integer*4 NPAD
	Logical UNITS
	Character*1 NULL, JDQ

	NULL = Char (0)
	JDQ = Char (34)

	NPAD = 0
	UNITS = .true.
	FDEG = FRAD / 1.74532925199433d-2
	Call HCSTR (FDEG,NDEC,NPAD,UNITS,STRING)

c  Remove trailing zeroes in seconds

	Call VLEN (STRING,LT)
	If (STRING(LT-4:LT-1) .eq. '.000') Then
	    STRING(LT-4:LT-4) = JDQ
	    STRING(LT-3:LT-3) = NULL
	    STRING(LT-2:LT-2) = NULL
	    STRING(LT-1:LT-1) = NULL
	    STRING(LT:LT) = NULL
	Elseif (STRING(LT-3:LT-1) .eq. '.00') Then
	    STRING(LT-3:LT-3) = JDQ
	    STRING(LT-2:LT-2) = NULL
	    STRING(LT-1:LT-1) = NULL
	    STRING(LT:LT) = NULL
	Elseif (STRING(LT-2:LT-1) .eq. '.0') Then
	    STRING(LT-2:LT-2) = JDQ
	    STRING(LT-1:LT-1) = NULL
	    STRING(LT:LT) = NULL
	Endif

c  Remove arcseconds if zero

	Call VLEN (STRING,LT)
	If (STRING(LT-2:LT-1) .eq. '00') Then
	    STRING(LT-2:LT-2) = NULL
	    STRING(LT-1:LT-1) = NULL
	    STRING(LT:LT) = NULL
	    Endif

c  Remove arcminutes if zero

	Call VLEN (STRING,LT)
	If (STRING(LT-2:LT-1) .eq. '00') Then
	    STRING(LT-2:LT-2) = NULL
	    STRING(LT-1:LT-1) = NULL
	    STRING(LT:LT) = NULL
	    Endif

	Return
	End


c--- HCSTRC  Convert declination to colon-separated string; drop trailing zeroes

	Subroutine HCSTRC (FRAD,NDEC,STRING)

	Real*8 FRAD
c			Fractional degrees or hours
	Integer*4 NDEC
c			Number of decimal places in seconds
	Character*(*) STRING
c			String (returned)
	Real*8 FDEG
	Integer*4 LT,I
	Integer*4 NPAD
	Logical UNITS
	Character*1 NULL, JDQ

	NULL = Char (0)
	JDQ = Char (34)

	NPAD = 1
	UNITS = .false.
	FDEG = FRAD / 1.74532925199433d-2
	Call HCSTR (FDEG,NDEC,NPAD,UNITS,STRING)

c  Remove trailing zeroes in seconds

	Call VLEN (STRING,LT)
	If (STRING(LT-4:LT-1) .eq. '.000') Then
	    STRING(LT-4:LT-4) = JDQ
	    STRING(LT-3:LT-3) = NULL
	    STRING(LT-2:LT-2) = NULL
	    STRING(LT-1:LT-1) = NULL
	    STRING(LT:LT) = NULL
	Elseif (STRING(LT-3:LT-1) .eq. '.00') Then
	    STRING(LT-3:LT-3) = JDQ
	    STRING(LT-2:LT-2) = NULL
	    STRING(LT-1:LT-1) = NULL
	    STRING(LT:LT) = NULL
	Elseif (STRING(LT-2:LT-1) .eq. '.0') Then
	    STRING(LT-2:LT-2) = JDQ
	    STRING(LT-1:LT-1) = NULL
	    STRING(LT:LT) = NULL
	Endif

c  Remove arcseconds if zero

	Call VLEN (STRING,LT)
	If (STRING(LT-2:LT-1) .eq. '00') Then
	    STRING(LT-2:LT-2) = NULL
	    STRING(LT-1:LT-1) = NULL
	    STRING(LT:LT) = NULL
	    Endif

c  Remove arcminutes if zero

	Call VLEN (STRING,LT)
	If (STRING(LT-2:LT-1) .eq. '00') Then
	    STRING(LT-2:LT-2) = NULL
	    STRING(LT-1:LT-1) = NULL
	    STRING(LT:LT) = NULL
	    Endif

c  Replace spaces with colons

	Call VLEN (STRING,LT)
	STRING(LT:LT) = NULL
	Do I = 2, LT-1
	    If (STRING(I:I) .eq. ' ') STRING(I:I) = ':'
	    Enddo

	Return
	End


c--- HCSTRH  Convert declination to vigesimal string; drop trailing zeroes

	Subroutine HCSTRH (FRAD,NDEC,STRING)

	Real*8 FRAD
c			Fractional degrees or hours
	Integer*4 NDEC
c			Number of decimal places in seconds
	Character*(*) STRING
c			String (returned)
	Integer*4 IC, I, LSTR
	Real*8 FDEG

	FDEG = FRAD / 1.74532925199433d-2
	Call HCSTRC (FRAD,NDEC,STRING)

c  Change first : to . ; return if there is none

	IC = Index (STRING,':')
	If (IC .le. 0) Return
	STRING(IC:IC) = '.'

c  Delete second :

	IC = Index (STRING,':')
	If (IC .gt. 0) Then
	    Call VLEN (STRING,LSTR)
	    STRING(IC:IC) = STRING(IC+1:IC+1)
	    STRING(IC+1:IC+1) = STRING(IC+2:IC+2)

c  If there are decimal seconds, delete .

	    If (STRING(IC+2:IC+2) .eq. '.') Then
		Do I = IC+3, LSTR
		    STRING(I-2:I-2) = STRING(I:I)
		    Enddo
		STRING(LSTR-1:LSTR-1) = '$'
	    Else
		STRING(IC+2:IC+2) = '$'
	    Endif
	    Call PADNULL (STRING)
	    Endif

	Return
	End


c--- HCSTRS  Convert declination to seconds string and drop leading spaces

	Subroutine HCSTRS (FRAD,NDEC,STRING)

	Real*8 FRAD
c			Angle in radians
	Integer*4 NDEC
c			Number of decimal places in seconds
	Character*(*) STRING
c			String (returned)
	Real*8 SDEC
	Integer*4 LSTR, ND, NF, LNUM, LSP, I, ISEC
	Character*16 FRMT

	Include 'constants.cmb'

c  Convert from radians to arcseconds

	SDEC = 3.6d3 * FRAD / RCON

c  Maximum length of output string

	LSTR = Len (STRING)

c  Set length out output string (include sign and decimal point)

	NF = NDEC
	If (SDEC .ge. 0) Then
	    ND = Int (Dlog10 (SDEC)) + 1
	Else
	    ND = Int (Dlog10 (-SDEC)) + 1
	Endif
	If (ND .lt. 1) ND = 1
	If (NDEC .gt. 0) Then
	    LNUM = NDEC + ND + 2
	Else
	    LNUM = NDEC + ND
	Endif
	If (LNUM .gt. LSTR) Then
	    NF = NDEC - (LNUM - LSTR)
	    If (NF .lt. 0) Then
		Do I = 1, LSTR-1
		    STRING(I:I) = '*'
		    Enddo
		STRING(LSTR:LSTR) = Char (0)
		Return
		Endif
	    LNUM = LSTR
	Else
	    NF = NDEC
	Endif

c  Set up output format

	If (NF .eq. 0) Then
	    If (LNUM .lt. 10) Then
		Write(FRMT,10) LNUM
10		Format('(I',I1,')')
	    Else
		Write(FRMT,11) LNUM
11		Format('(I',I2,')')
	    Endif
	Elseif (NF .lt. 10) Then
	    If (LNUM .lt. 10) Then
		Write(FRMT,20) LNUM, NF
20		Format('(F',I1,'.',I1,')')
	    Else
		Write(FRMT,21) LNUM, NF
21		Format('(F',I2,'.',I1,')')
	    Endif
	Else
	    Write(FRMT,30) LNUM, NF
30	    Format('(F',I2,'.',I2,')')
	Endif

c  Write number to string

	If (NF .gt. 0) Then
	    Write(STRING,FRMT) SDEC
	Else
	    ISEC = Idnint (SDEC)
	    Write(STRING,FRMT) ISEC
	Endif

c  Remove leading spaces

	LSP = 0
	Do I = 1, LNUM
	    If (STRING(I:I) .eq. ' ') LSP = LSP + 1
	    Enddo
	If (LSP .gt. 0) Then
	    LNUM = LNUM - LSP
	    Do I = 1, LNUM
		STRING(I:I) = STRING(I+LSP:I+LSP)
		Enddo
	    STRING(LNUM+1:LNUM+1) = '$'
	    Endif
	Call PADNULL (STRING)

c  Remove hanging decimal point
	Call VLEN (STRING, LSTR)
	If (STRING(LSTR:LSTR) .eq. '.') Then
	    STRING(LSTR:LSTR) = Char (0)
	    Endif

	Return
	End

c Sep  2 1992	Convert DEC to degrees before calling HCON programs
c Oct 16 1992	Declare undeclared variables
c Nov 18 1992	Fix bug in RCONH

c Jan 26 1993	Add DCONF to convert from degrees to dd.mmsss

c Feb  7 1995	Add DCSTRT to drop unused part of strings
c Feb  8 1995	Add DCSTRC to drop unused part of strings with colons
c Sep 18 1995	Add DCSTRS to produce seconds string
c Dec  8 1995	Add DCSTRH to produce vigesimal string
c Dec  8 1995	Remove hanging decimal point in seconds string

c Apr 14 1988	Fix roundoff error in HCON1 and HCON2
c May 10 1988	Add HCONR to return angle in radians

c May 17 1990	Fix HCON1 to retain precision
c Aug  8 1990	Add HCONS radians to dd.mmsss

c Feb 22 1993	Make conversions explicit

c Dec 12 1995	Add trailing-zero-eliminating options

c Dec  1 2004	Include only from local directory
