c*** File iolib/jcon.f
c*** April 10, 1998
c*** By Doug Mink

c  JCON (DATE,TIME,DJ)
c	Convert date as yyyy.ddmm and time as hh.mmsss to Julian Date

c  JCON1 (DJ,DATE,TIME)
c	Convert Julian Date to date as yyyy.ddmm and time as hh.mmsss

c  JCONS (DJ,STRING)
c	Convert Julian Date to year day month hours:minutes:seconds string

c  JCON2 (DJ,IYR,IMON,IDAY,IHR,IMN,SEC)
c	Convert Julian Date to year month day hours minutes seconds .1ms

c  JCONY (YEAR,DAY,TIME,DJ)
c	Convert year, day, and time as hh.mmsss to Julian Date

c  JCONV (DJ,TSEC)
c	Convert Julian Day to seconds since 1950.0

c  JCONM (DJ,MJD)
c	Convert Julian Date to Modified Julian Date

c  JCONT (TSEC,DJ)
c	Convert seconds since 1950.0 to Julian Day


c  JCON-- Convert from date as yyyy.mmdd and time as hh.mmsss to Julian Date

	Subroutine JCON (DATE,TIME,DJ)

	Real*8 DATE
c			Date as yyyy.mmdd
c			    yyyy = calendar year (e.g. 1973)
c			    mm = calendar month (e.g. 04 = april)
c			    dd = calendar day (e.g. 15)
	Real*8 TIME
c			Time as hh.mmssxxxx
c		       *if TIME<0, it is time as -(fraction of a day)
c			    hh = hour of day (0 .le. hh .le. 23)
c			    nn = minutes (0 .le. nn .le. 59)
c			    ss = seconds (0 .le. ss .le. 59)
c			  xxxx = tenths of milliseconds (0 .le. xxxx .le. 9999)
	Real*8 DJ
c			Julian Date (returned)

	Real*8 TSEC

	If (DATE .lt. 0.0301d0) Return
	Call VCON (DATE,TIME,TSEC)
	DJ = 2433282.5d0 + (TSEC / 8.64d4)

	Return
	End


c  JCON1-- Convert Julian Date to date as yyyy.mmdd and time as hh.mmssss

	Subroutine JCON1 (DJ,DATE,TIME)

	Real*8 DJ
c			Julian Date
	Real*8 DATE
c			Date as yyyy.mmdd (returned)
c			    yyyy = calendar year (e.g. 1973)
c			    mm = calendar month (e.g. 04 = april)
c			    dd = calendar day (e.g. 15)
	Real*8 TIME
c			Time as hh.mmssxxxx (returned)
c		       *if TIME<0, it is time as -(fraction of a day)
c			    hh = hour of day (0 .le. hh .le. 23)
c			    nn = minutes (0 .le. nn .le. 59)
c			    ss = seconds (0 .le. ss .le. 59)
c			  xxxx = tenths of milliseconds (0 .le. xxxx .le. 9999)

	Real*8 TSEC

	TSEC = (DJ - 2433282.5d0) * 8.64d4
	Call VCON1 (TSEC,DATE,TIME)

	Return
	End


c  JCONE-- Convert Julian Date to fractional year as used in epoch

	Subroutine JCONE (DJ, EPOCH)

	Real*8 DJ
c			Julian Date
	Real*8 EPOCH
c			Date as fractional year (returned)

	Real*8 DJ0, DJ1, DATE0, TIME0, DATE1, DATE, TIME

	Call JCON1 (DJ, DATE, TIME)
	TIME0 = 12.d0
	DATE0 = Dint (DATE) + 0.0101d0
	DATE1 = Dint (DATE) + 1.0101d0
	Call JCON (DATE0, TIME0, DJ0)
	Call JCON (DATE1, TIME0, DJ1)
	EPOCH = Dint (DATE) + ((DJ - DJ0) / (DJ1 - DJ0))
	Return
	End


c  JCONS-- Convert Julian Date to year month day hours:minutes:seconds string

	Subroutine JCONS (DJ,STRING)

	Real*8 DJ
c				Julian Date
	Character*(*) STRING
c				Formatted date string (returned)

	Real*8 TSEC

	TSEC = (DJ - 2433282.5d0) * 8.64d4
	Call VCONS (TSEC,STRING)

	Return
	End


c  JCONP-- Convert Julian Date to day month year hours:minutes:seconds string

	Subroutine JCONP (DJ,STRING)

	Real*8 DJ
c				Julian Date
	Character*(*) STRING
c				Formatted date string (returned)

	Real*8 TSEC

	TSEC = (DJ - 2433282.5d0) * 8.64d4
	Call VCONP (TSEC,STRING)

	Return
	End


c  JCON2-- Convert Julian Date to year month day hours minutes seconds .1ms

	Subroutine JCON2 (DJ,IYR,IMON,IDAY,IHR,IMN,SEC)

	Real*8 DJ
c			Julian Date
	Integer*4 IYR
c			Year (returned)
	Integer*4 IMON
c			Month (returned)
	Integer*4 IDAY
c			Day (returned)
	Integer*4 IHR
c			Hours (returned)
	Integer*4 IMN
c			Minutes (returned)
	Real*8 SEC
c			Seconds (returned)

	Real*8 TSEC

	TSEC = (DJ - 2433282.5d0) * 8.64d4
	Call VCON2 (TSEC,IYR,IMON,IDAY,IHR,IMN,SEC)

	Return
	End


c JCONY-- Convert year, day, and time as hh.mmsss to Julian Date

	Subroutine JCONY (YEAR,DAY,TIME,DJ)

	Integer*4 YEAR
c			Year
	Integer*4 DAY
c			Day of the year (1 - 365 or 366)
	Real*8 TIME
c			Time as hh.mmssxxxx
c		       *if TIME<0, it is time as -(fraction of a day)
c			    hh = hour of day (0 .le. hh .le. 23)
c			    nn = minutes (0 .le. nn .le. 59)
c			    ss = seconds (0 .le. ss .le. 59)
c			  xxxx = tenths of milliseconds (0 .le. xxxx .le. 9999)
	Real*8 DJ
c			Julian Date (returned)

	Real*8 TSEC

	Call VCONY (YEAR,DAY,TIME,TSEC)
	DJ = 2433282.5d0 + (TSEC / 8.64d4)

	Return
	End


c  JCONM-- Convert Julian Date to Modified Julian Date

	Subroutine JCONM (DJ, MJD)

	Real*8 DJ
c				Julian Date
	Real*8 MJD
c				Modified Julian Date (returned)

	MJD = DJ - 2400000.5d0
	Return
	End


c  JCONV-- Convert Julian Date to seconds since 1950.0

	Subroutine JCONV (DJ,TSEC)

	Real*8 DJ
c				Julian Date
	Real*8 TSEC
c				Seconds since 1950.0 (returned)

	TSEC = (DJ - 2433282.5d0) * 8.64d4
	Return
	End


c  JCONT-- Convert seconds since 1950.0 to Julian Date

	Subroutine JCONT (TSEC,DJ)

	Real*8 TSEC
c				Seconds since 1950.0
	Real*8 DJ
c				Julian Date (returned)

	DJ = 2433282.5d0 + (TSEC / 8.64d4)
	Return
	End

c Jun 28 1994	New program

c Apr 17 1995	Add jcont to convert from seconds to Julian date

c Apr 10 1998	Add jcone to convert from Julian Date to fraction of year
