c*** File iolib/mjcon.f
c*** November 7, 2003
c*** By Doug Mink

c  MJCON (DATE,TIME,DJM)
c	Convert date as yyyy.ddmm and time as hh.mmsss to Modified Julian Date

c  MJCON1 (DJM,DATE,TIME)
c	Convert Modified Julian Date to date as yyyy.ddmm and time as hh.mmsss

c  MJCONS (DJM,STRING)
c	Convert Modified Julian Date to year day month hours:minutes:seconds string

c  MJCON2 (DJM,IYR,IMON,IDAY,IHR,IMN,SEC)
c	Convert Modified Julian Date to year month day hours minutes seconds .1ms

c  MJCONY (YEAR,DAY,TIME,DJM)
c	Convert year, day, and time as hh.mmsss to Modified Julian Date

c  MJCONV (DJM,TSEC)
c	Convert Modified Julian Day to seconds since 1950.0

c  MJCONT (TSEC,DJM)
c	Convert seconds since 1950.0 to Modified Julian Day


c  MJCON-- Convert from date as yyyy.mmdd and time as hh.mmsss to Julian Date

	Subroutine MJCON (DATE,TIME,DJM)

	Real*8 DATE
c			Date as yyyy.mmdd
c			    yyyy = calendar year (e.g. 1973)
c			    mm = calendar month (e.g. 04 = april)
c			    dd = calendar day (e.g. 15)
	Real*8 TIME
c			Time as hh.mmssxxxx
c		       *if TIME<0, it is time as -(fraction of a day)
c			    hh = hour of day (0 .le. hh .le. 23)
c			    nn = minutes (0 .le. nn .le. 59)
c			    ss = seconds (0 .le. ss .le. 59)
c			  xxxx = tenths of milliseconds (0 .le. xxxx .le. 9999)
	Real*8 DJM
c			Modified Julian Date (returned)

	Call JCON (DATE, TIME, DJM)
	DJM = DJM - 2400000.5d0

	Return
	End


c  MJCON1-- Convert Modified Julian Date to date as yyyy.mmdd and time as hh.mmssss

	Subroutine MJCON1 (DJM,DATE,TIME)

	Real*8 DJM
c			Modified Julian Date
	Real*8 DATE
c			Date as yyyy.mmdd (returned)
c			    yyyy = calendar year (e.g. 1973)
c			    mm = calendar month (e.g. 04 = april)
c			    dd = calendar day (e.g. 15)
	Real*8 TIME
c			Time as hh.mmssxxxx (returned)
c		       *if TIME<0, it is time as -(fraction of a day)
c			    hh = hour of day (0 .le. hh .le. 23)
c			    nn = minutes (0 .le. nn .le. 59)
c			    ss = seconds (0 .le. ss .le. 59)
c			  xxxx = tenths of milliseconds (0 .le. xxxx .le. 9999)

	Call JCON1 (DJM + 2400000.5d0, DATE, TIME)

	Return
	End


c  MJCONE-- Convert Modified Julian Date to fractional year as used in epoch

	Subroutine MJCONE (DJM, EPOCH)

	Real*8 DJM
c			Modified Julian Date
	Real*8 EPOCH
c			Date as fractional year (returned)

	Call JCONE (DJM+2400000.5d0, EPOCH)
	Return
	End


c  MJCONS-- Convert Modified Julian Date to year month day hours:minutes:seconds string

	Subroutine MJCONS (DJM,STRING)

	Real*8 DJM
c				Modified Julian Date
	Character*(*) STRING
c				Formatted date string (returned)

	Call JCONS (DJM+2400000.5d0, STRING)

	Return
	End


c  MJCONP-- Convert Modified Julian Date to day month year hours:minutes:seconds string

	Subroutine MJCONP (DJM, STRING)

	Real*8 DJM
c				Modified Julian Date
	Character*(*) STRING
c				Formatted date string (returned)

	Call JCONP (DJM + 2400000.5d0, STRING)

	Return
	End


c  MJCON2-- Convert Modified Julian Date to year month day hours minutes seconds .1ms

	Subroutine MJCON2 (DJM,IYR,IMON,IDAY,IHR,IMN,SEC)

	Real*8 DJM
c			Modified Julian Date
	Integer*4 IYR
c			Year (returned)
	Integer*4 IMON
c			Month (returned)
	Integer*4 IDAY
c			Day (returned)
	Integer*4 IHR
c			Hours (returned)
	Integer*4 IMN
c			Minutes (returned)
	Real*8 SEC
c			Seconds (returned)

	Call JCON2 (DJM + 2400000.5d0, IYR,IMON,IDAY,IHR,IMN,SEC)

	Return
	End


c MJCONY-- Convert year, day, and time as hh.mmsss to Modified Julian Date

	Subroutine MJCONY (YEAR,DAY,TIME,DJM)

	Integer*4 YEAR
c			Year
	Integer*4 DAY
c			Day of the year (1 - 365 or 366)
	Real*8 TIME
c			Time as hh.mmssxxxx
c		       *if TIME<0, it is time as -(fraction of a day)
c			    hh = hour of day (0 .le. hh .le. 23)
c			    nn = minutes (0 .le. nn .le. 59)
c			    ss = seconds (0 .le. ss .le. 59)
c			  xxxx = tenths of milliseconds (0 .le. xxxx .le. 9999)
	Real*8 DJM
c			Modified Julian Date (returned)

	Call JCONY (YEAR,DAY,TIME,DJM)
	DJM = DJM - 2400000.5d0

	Return
	End


c  MJCONV-- Convert Modified Julian Date to seconds since 1950.0

	Subroutine MJCONV (DJ,TSEC)

	Real*8 DJ
c				Julian Date
	Real*8 TSEC
c				Seconds since 1950.0 (returned)

	TSEC = (DJM  + 2400000.5d0 - 2433282.5d0) * 8.64d4
	Return
	End


c  MJCONT-- Convert seconds since 1950.0 to Julian Date

	Subroutine MJCONT (TSEC,DJM)

	Real*8 TSEC
c				Seconds since 1950.0
	Real*8 DJM
c				Modified Julian Date (returned)

	DJM = 2433282.5d0 - 2400000.5d0 + (TSEC / 8.64d4)
	Return
	End

c Oct 31 2003	New programs based on those in jcon.c
c Nov  7 2003	Call JCONi subroutines
