c*** File iolib/pget.f
c*** May 3, 1993
c*** By Doug Mink

c--- Subroutines to extract values for parameters from a parameter string

c	PGET   (HSTRING,VNAME,ITYPE,IVAL) returns any type
c	PGETI2 (HSTRING,VNAME,IVAL) returns Integer*2 IVAL
c	PGETI4 (HSTRING,VNAME,IVAL) returns Integer*4 IVAL
c	PGETR4 (HSTRING,VNAME,RVAL) returns Real*4 RVAL
c	PGETR8 (HSTRING,VNAME,DVAL) returns Real*8 DVAL
c	PGETL  (HSTRING,VNAME,LVAL) returns Logical LVAL
c	PGETUP (HSTRING,VNAME,CVAL) returns Character string CVAL as upper case
c	PGETC  (HSTRING,VNAME,CVAL) returns Character string CVAL
c	PGETV  (HSTRING,VNAME) prints current parameter value as in verbose
c	PGETCOM (COMMENT) Return comment for last parameter gotten from string

c--- PGET  Extract value for parameter from parameter string (any type)

	Subroutine PGET (HSTRING,VNAME,ITYPE,IVAL)

	Character*(*) HSTRING
c		Character string containing FITS-style header information
c		in the format <VNAME>= <value> {/ <comment>}
c		Unlike FITS, however, lines may be of arbitrary length
c		terminated by nulls, carriage returns or linefeeds,
c		although FITS' 80-column format is supported, too.

	Character*(*) VNAME
c		Character string containing the name of the variable
c		to be returned.  PGET searches for a line beginning
c		with this string.
c		(THE FIRST 8 CHARACTERS OF A VARIABLE NAME MUST BE UNIQUE)

	Integer*4 ITYPE
c		Data type
c		  first ':' treated as '.', later ':'s and '.'s ignored.
c		  illegal characters, overflows, and underflows trapped. 
c		-1n: Integer n bytes long (Integer*n)
c		-2n: Floating point number n bytes long (Real*n)
c		-3n: Logical variable n bytes long (Logical*n) (t or f)
c		-4n: Logical variable n bytes long (Logical*n) (y or n)
c		1-99: Length of Character string returned, padded with spaces
c		101-199: Length (+100) of Character string returned
c		        padded with nulls
c		201-236: number (+200) of single characters
c			returned in array of 36 (A-Z,0-9) Integer*2
c			with array positions corresponding to letter
c			or number entered set to one.  Array is not
c			zeroed on entry.
c			Lower case letters are treated as upper case.
c		301-399: Length (+300) of Character string returned
c		        padded with nulls
c			Lower case letters are converted to upper case.

c	(Integer,Real) IVAL
c		Address for returned number, character, array, or string
c		If a Character string is to be returned, call with
c		Equivalence'd Integer; otherwise, Fortran adds another
c		invisible string length to the calling parameter list
c		and messes up everything.

	Character*30 VALUE
c		Character string containing value from header

	Integer*2 IVAL(*)
	Integer*4 NCVAL,IER
	Include 'header.cmb'

c  Translate value from binary to ASCII

	Call BINASC (NCVAL,VALUE,ITYPE,IVAL,IER)

c  Get ASCII parameter value from header string

	Call PGETC (HSTRING,VNAME,VALUE)

c  Translate value from ASCII to binary

	Call ASCBIN (NCVAL,VALUE,ITYPE,IVAL,IER)

	Return
	End


c--- PGETI4  Extract Integer*4 value for variable from parameter string

	Subroutine PGETI4 (HSTRING,VNAME,IVAL)

	Character*(*) HSTRING
	Character*(*) VNAME
	Integer*4 IVAL

	Character*30 VALUE
	Integer*4 NCVAL,IER
	Include 'header.cmb'

c  Translate value from binary to ASCII

	Call BINASCI4 (IVAL,VALUE)

c  Get ASCII parameter value from header string

	Call PGETC (HSTRING,VNAME,VALUE)

c  Translate value from ASCII to binary

	Call VLEN (VALUE,NCVAL)
	Call ASCBINI4 (NCVAL,VALUE,IVAL,IER)

	Return
	End


c--- PGETI2  Extract Integer*2 value for variable from parameter string

	Subroutine PGETI2 (HSTRING,VNAME,IVAL)

	Character*(*) HSTRING
	Character*(*) VNAME
	Integer*2 IVAL

	Character*30 VALUE
	Integer*4 NCVAL,IER
	Include 'header.cmb'

c  Translate value from binary to ASCII

	Call BINASCI2 (IVAL,VALUE)

c  Get ASCII parameter value from header string

	Call PGETC (HSTRING,VNAME,VALUE)

c  Translate value from ASCII to binary

	Call VLEN (VALUE,NCVAL)
	Call ASCBINI2 (NCVAL,VALUE,IVAL,IER)

	Return
	End


c--- PGETR4  Extract Real*4 value for variable from parameter string

	Subroutine PGETR4 (HSTRING,VNAME,RVAL)

	Character*(*) HSTRING
	Character*(*) VNAME
	Real*4 RVAL

	Character*30 VALUE
	Integer*4 NCVAL,IER
	Include 'header.cmb'

c  Translate value from binary to ASCII

	Call BINASCR4 (RVAL,VALUE)

c  Get ASCII parameter value from header string

	Call PGETC (HSTRING,VNAME,VALUE)

c  Translate value from ASCII to binary

	Call VLEN (VALUE,NCVAL)
	Call ASCBINR4 (NCVAL,VALUE,RVAL,IER)

	Return
	End


c--- PGETR8  Extract Real*8 value for variable from parameter string

	Subroutine PGETR8 (HSTRING,VNAME,DVAL)

	Character*(*) HSTRING
	Character*(*) VNAME
	Real*8 DVAL

	Character*30 VALUE
	Integer*4 NCVAL,IER
	Include 'header.cmb'

c  Translate value from binary to ASCII

	Call BINASCR8 (DVAL,VALUE)

c  Get ASCII parameter value from header string

	Call PGETC (HSTRING,VNAME,VALUE)

c  Translate value from ASCII to binary

	Call VLEN (VALUE,NCVAL)
	Call ASCBINR8 (NCVAL,VALUE,DVAL,IER)

	Return
	End


c--- PGETL  Extract Logical value for variable from parameter string

	Subroutine PGETL (HSTRING,VNAME,LVAL)

	Character*(*) HSTRING
	Character*(*) VNAME
	Logical LVAL

	Character*1 VALUE
	Integer*4 IER
	Include 'header.cmb'

c  Translate value from binary to ASCII

	Call BINASCL (LVAL,VALUE)

c  Get ASCII parameter value from header string

	Call PGETC (HSTRING,VNAME,VALUE)

c  Translate value from ASCII to binary

	Call ASCBINL (VALUE,LVAL,IER)

	Return
	End


c--- PGETUP Extract upper case character value from parameter string

	Subroutine PGETUP (HSTRING,VNAME,CVAL)

	Character*(*) HSTRING
	Character*(*) VNAME
	Character*(*) CVAL

	Integer*4 ILEN,ICH,I
	Include 'header.cmb'

c  Get ASCII parameter value from header string

	Call PGETC (HSTRING,VNAME,CVAL)

c  Make sure that string is all upper case

	Call VLEN (CVAL,ILEN)
	Do 100 I = 1, ILEN
	    ICH = Ichar (CVAL(I:I))
	    If ((ICH .gt. 96) .and. (ICH .lt. 123)) Then
		CVAL(I:I) = Char (ICH-32)
		Endif
100	    Continue

	Return
	End


c--- PGETV  Extract value for variable from parameter string and print it

	Subroutine PGETV (HSTRING,VNAME)

	Character*(*) HSTRING
	Character*(*) VNAME

	Logical VB
	Character*64 VALUE
	Include 'header.cmb'

c  Get ASCII parameter value from header string

	VB = VERBOSE
	VERBOSE = .true.
	Call ZSTRING (VALUE)
	Call PGETC (HSTRING,VNAME,VALUE)
	VERBOSE = VB

	Return
	End


c--- PGETC  Extract Character value for variable from parameter string

	Subroutine PGETC (HSTRING,VNAME,CVAL)

	Character*(*) HSTRING
	Character*(*) VNAME
	Character*(*) CVAL
c			Character string containing the value for variable
c			VNAME.  Trailing and leading blanks are removed.

	Character*100 LINE
	Integer*4 LVNAME,LCOM,NCVAL,LLIN,LCVAL
	Integer*4 L0,L1,LQ0,LQ1,LC0,LC1,LC,LT0,LV0,LV1,LV,NCOM

	Include 'header.cmb'
	Include 'prompt.cmb'

	FOUND = .false.

c  Find length of variable name

	Call VLEN (VNAME,LVNAME)

c  Search header string for variable name

	Call PSEARCH (HSTRING,VNAME,L0,LLIN)

c  If not found, drop out of subroutine

	If (L0 .le. 0) Then
	    If (VERBOSE) Then
		Call VLEN (CVAL,LCVAL)
		Write(LINE,31) VNAME(1:LVNAME),CVAL(1:LCVAL)
31		Format(A,' = ',A,' [default]$')
		Call PSTRING (LINE)
		Endif
	    Call ZSTRING (PCOM)
	    Return
	    Endif

c  Set flag indicating that the variable was found in the header

	FOUND = .true.

c  Entry lasts until linefeed

	L1 = L0 + LLIN - 1

c  Extract entry for this variable from the header

	LINE = HSTRING(L0:L1)
	Call PADNULL (LINE)

c  Check for quoted value (both single and double quotes allowed)

	LQ0 = Index (LINE,Char(39))
	LC0 = Index (LINE,'/')
	If (LQ0 .gt. 0) Then
	    If ((LC0 .le. 0) .or. (LC0 .gt. LQ0)) Then
		LQ1 = Index (LINE(LQ0+1:),Char(39)) + LQ0
		LC0 = Index (LINE(LQ1+1:),'/')
		If (LC0 .gt. 0) LC0 = LQ1 + LC0
	    Else
		LQ0 = 0
		LQ1 = 0
	    Endif
	Else
	    LQ0 = Index (LINE,'"')
	    If (LQ0 .gt. 0) Then
		If ((LC0 .ge. 0) .or. (LC0 .gt. LQ0)) Then
		    LQ1 = Index (LINE(LQ0+1:),'"') + LQ0
		    LC0 = Index (LINE(LQ1+1:),'/')
		    If (LC0 .gt. 0) LC0 = LQ1 + LC0
		Else
		    LQ0 = 0
		    LQ1 = 0
		Endif
	    Else
		LQ1 = 0
	    Endif
	Endif

c  Check for tabs after parameter value

	LT0 = Index (LINE(LQ1+1:),Char(9))
	If (LT0 .gt. 0) LT0 = LT0 + LQ1

c  Find start of comment field

	If (LT0 .gt. 0) Then
	    LC0 = Index (LINE(LT0+1:),'/')
	    If (LC0 .gt. 0) LC0 = LC0 + LT0
	Else
	    LC0 = Index (LINE(LQ1+1:),'/')
	    If (LC0 .gt. 0) LC0 = LC0 + LQ1
	Endif

c  Extract value from line

	If (LQ0 .gt. 0) Then
	    LV0 = LQ0 + 1
	    LV1 = LQ1 - 1
	Else
	    LV0 = Index (LINE,'=') + 1
	    LV1 = LT0 - 1
	    If (LV1 .lt. 0) LV1 = LC0 - 1
	    If (LV1 .lt. 0) LV1 = LLIN
	Endif

c  Remove leading and trailing spaces from value string

	LCVAL = Len (CVAL)
	Call ZSTRING (CVAL)
	Do 100 LV = LV0, LV1
100	    If (LINE(LV:LV) .ne. ' ') Go to 110
	NCVAL = 0
	Go to 150
110	LV0 = LV
	Do 120 LV = LV1, LV0, -1
120	    If (LINE(LV:LV) .ne. ' ') Go to 130
130	LV1 = LV
	NCVAL = LV1 - LV0 + 1
	If (LCVAL .lt. NCVAL) NCVAL = LCVAL
	CVAL(1:NCVAL) = LINE(LV0:LV1)

c  Extract comment and remove excess spaces

150	Call ZSTRING (PCOM)
	If ((LC0 .gt. 0) .and. (LC0 .lt. LLIN)) Then
	    LC0 = LC0 + 1
	    LC1 = LLIN
	    Do 200 LC = LC0, LC1
200		If (LINE(LC:LC) .ne. ' ') Go to 210
	    Go to 240
210	    LC0 = LC
	    Do 220 LC = LC1, LC0, -1
220		If (LINE(LC:LC) .ne. ' ') Go to 230
230	    LC1 = LC
	    NCOM = LC1 - LC0 + 1
	    PCOM(1:NCOM) = LINE(LC0:LC1)
	    Endif

240	If (VERBOSE) Then
	    Call VLEN (PCOM,LCOM)
c	    Write(*,'(A)') LINE(1:LLIN)
c	    Write(*,*) 'Quotes:',LQ0,'-',LQ1,'Tabs from',LT0
c	    Write(*,*) 'Value:',LV0,'-',LV1,NCVAL,'Comment:',LC0,'-',LC1,LCOM
	    If (LCOM .gt. 0) Then
		Write(LINE,250) VNAME(1:LVNAME),CVAL(1:NCVAL),PCOM(1:LCOM)
250		Format(A,' = ',A,' [',A,']$')
	    Else
		Write(LINE,255) VNAME(1:LVNAME),CVAL(1:NCVAL)
255		Format(A,' = ',A,'$')
	    Endif
	    Call PSTRING (LINE)
	    Endif
	    
	Return
	End


c--- PGETCOM  Return comment for last parameter that was read

	Subroutine PGETCOM (COMMENT)

	Character*(*) COMMENT
c		Character string containing comment from header

	Include 'prompt.cmb'

	COMMENT = PCOM

	Call PADNULL (COMMENT)

	Return
	End

c Nov  2 1987	Write VERBOSE diagnostics using PSTRING
c Dec 14 1987	Print default value if parameter is not found

c Jan 22 1988	Replace PADSTR call with PADNULL
c Apr  6 1988	Add self-documenting comments

c Apr 11 1989	Add ASCBINR8 and ASCBINR4 for RISC machine

c Mar 14 1990	Fix quotes and comments
c May 15 1990	Repair fix to quotes and comments
c Sep 28 1990	Remove unreferenced variables

c Jun 28 1991	Clean up comment handling

c Sep  8 1992	Call ASCBINL instead of ASCBIN
c Sep 10 1992	Call ASCBINI2 instead of ASCBIN
c Oct 10 1992	Add TGETNC to get character strings with no parameter file

c Feb 22 1993	Eliminate unused variable null
c May  3 1993	Make verbose mode clearer using brackets

c Jul  6 1994	Fix bug to ignore slashes inside quotes
