c*** File iolib/smooth.f
c*** June 20, 1997
c*** By Doug Mink, Center for Astrophysics

c--- Smooth a data data vector

        Subroutine SMOOTH (DATAVEC, NPTS, NDSAMP0, DHALF, NSTYPE)

	Real*4 DATAVEC(*)
c				Data vector (returned smoothed)
	Integer*4 NPTS
c				Number of pixels in vector
	Integer*4 NDSAMP0
c				No. of 121 smoothings or points in Gaussian
	Real*8 DHALF
c				Half width of Gaussian in pixels
	Integer*4 NSTYPE
c				Processing (0=smooth, 1=Gaussian, 2=Hanning)

	Integer*4 NDSAMP2
c				Half of NDSAMP
	Integer*4 NDSAMP
c				No. of 121 smoothings or width of Gaussian
	Real*8 WT(100)
c				Gaussian weighting function per pixel
	Real*8 WVEC(100)
c				Vector for points in Gaussian
	Real*8 WTPIX
c				Gaussian weighting function total

	Real*8 TWOPI,SIG2,RAD2,SUMDATA
	Real*4 SPJ,DATA0,DATA1,DATA2
	Integer*4 IPIXN
	Integer*4 ISAMP,I,IPIX,JWT

	If ((NSTYPE .eq. 0) .and. (NDSAMP .le. 0)) Return

c  Repeated 1-2-1 smoothing
	If (NSTYPE .eq. 0) Then
	    Do I =1,NDSAMP
		DATA0 = DATAVEC(1)
		DATA1 = DATAVEC(1)
		Do IPIX = 1, NPTS
		    IPIXN = IPIX + 1
		    If (IPIXN .gt. NPTS) IPIXN = NPTS
		    DATA2 = DATAVEC(IPIXN)
		    DATAVEC(IPIX) = (DATA0 + DATA1 + DATA1 + DATA2) / 4.0
		    DATA0 = DATA1
		    DATA1 = DATA2
		    Enddo
		Enddo

c  Hanning or Gaussian smoothing
	Else
	    TWOPI = 6.2831853
	    SIG2 = 2.d0 * DHALF * DHALF

c  Set up weighting functions for Gaussian and Hanning smoothing
	    NDSAMP2 = NDSAMP0 / 2
	    NDSAMP = 1 + (NDSAMP2 * 2)
	    WTPIX = 0.d0
	    Do ISAMP = -NDSAMP2, NDSAMP2
		I = ISAMP + NDSAMP2 + 1
		RAD2 = Dble (ISAMP) * Dble (ISAMP)
		If (NSTYPE .eq. 2) Then
		    WT(I) = 0.5 * (1.0 - Dcos (TWOPI*Dble(I) / Dble(NDSAMP-1)))
		Else
		    WT(I) = Exp (-RAD2 / SIG2)
		Endif
		WTPIX = WTPIX + WT(I)
		Enddo
	    if (WTPIX .le. 0.d0) WTPIX = 1.d0

c  Initialize vector of points in Gaussian
	    IPIX = 1
	    WTPIX = 0.d0
	    Do ISAMP = -NDSAMP2, NDSAMP2
		If (ISAMP .le. 0) Then
		     WVEC(IPIX) = DATAVEC(1)
		Else
		     WVEC(IPIX) = DATAVEC(ISAMP+1)
		Endif
		IPIX = IPIX + 1
		Enddo

c  For each point in input vector, compute smoothed value
	    Do IPIX = 1, NPTS
		SUMDATA = 0.d0
		WTPIX = 0.d0
		Do JWT = 1, NDSAMP
		    SUMDATA = SUMDATA + WT(JWT) * WVEC(JWT)
		    Enddo
		SPJ = Sngl (SUMDATA / WTPIX)
		DATAVEC(IPIX) = SPJ

c  Shift values in computing vector so it is center on the next data point
		Do ISAMP = 1, NDSAMP-1
		    WVEC(I) = WVEC(I+1)
		    Enddo
		If (IPIX + NDSAMP2 .le. NPTS) Then
		    WVEC(NDSAMP) = DATAVEC(IPIX+NDSAMP2)
		    Endif
		Enddo
	Endif

	Return
	End

c Jun 20 1997	New subroutine based on SPECSMOOTH in TDCLIB
