c*** File iolib/tget.f
c*** September 14, 1995
c*** By Doug Mink

	Subroutine TGET (HSTRING,PROMPT,VNAME,ITYPE,IVAL)

c  Prompt and read value for parameter VNAME from terminal, reading the default
c  value (and optionally the prompt string) from parameter string HSTRING,
c  and returning the new value to address IVAL, as well as modifying the
c  parameter string.

c	TGET    (HSTRING,PROMPT,VNAME,ITYPE,IVAL) returns any type
c	TGETA	(HSTRING,PROMPT,ITYPE,VNAME) only modifies parameter string
c	TGETC   (HSTRING,PROMPT,VNAME,CVAL) returns all-caps Character string CVAL
c	TGETCHAR (PROMPT,CHAR) returns single character
c	TGETCOM (PROMPT,CCOM) returns single character CCOM
c	TGETI2  (HSTRING,PROMPT,VNAME,IVAL) returns Integer*2 IVAL
c	TGETI4  (HSTRING,PROMPT,VNAME,IVAL) returns Integer*4 IVAL
c	TGETL   (HSTRING,PROMPT,VNAME,LVAL) returns Logical LVAL
c	TGETN   (PROMPT,ITYPE,IVAL) returns any type, no string modification
c	TGETNC  (PROMPT,ITYPE,CVAL) returns character string, no string mod
c	TGETR4  (HSTRING,PROMPT,VNAME,RVAL) returns Real*4 RVAL
c	TGETR8  (HSTRING,PROMPT,VNAME,DVAL) returns Real*8 DVAL
c	TGETS   (HSTRING,PROMPT,VNAME,CVAL) returns Character string CVAL
c	TPROG   (PROGNAME) sets program name for prompt
c	TPGET   (PROGNAME) gets program name used for prompt
c	TPNAM   () adds program name to next prompt
c	TPUT    (STRING)  writes string to terminal preceded by command prompt
c	TPROMPT (PROMPT,VNAME) makes prompt in several ways

	Character*(*) HSTRING
c		Character string containing FITS-style header information
c		in the format <VNAME>= <value> {/ <comment>}
c		Unlike FITS, however, lines may be of arbitrary length
c		terminated by nulls, carriage returns or linefeeds,
c		although FITS' 80-column format is supported, too.

	Character*(*) PROMPT
c		Prompt to be printed at terminal
c		If PROMPT is null, use comment from parameter file.
c		If there is no comment, prompt with parameter name only.

	Character*(*) VNAME
c		Character string containing the name of the variable
c		to be returned.  TGET searches for a line beginning
c		with this string.
c		(THE FIRST 8 CHARACTERS OF A VARIABLE NAME MUST BE UNIQUE)

	Integer*4 ITYPE
c		Data type
c		  first ':' treated as '.', later ':'s and '.'s ignored.
c		  illegal characters, overflows, and underflows trapped. 
c		-1n: Integer n bytes long (Integer*n)
c		-2n: Floating point number n bytes long (Real*n)
c		-3n: Logical variable n bytes long (Logical*n) (t or f)
c		-4n: Logical variable n bytes long (Logical*n) (y or n)
c		1-99: Length of Character string returned, padded with spaces
c		101-199: Length (+100) of Character string returned
c		        padded with nulls
c		201-236: number (+200) of single characters
c			returned in array of 36 (A-Z,0-9) Integer*2
c			with array positions corresponding to letter
c			or number entered set to one.  Array is not
c			zeroed on entry.
c			Lower case letters are treated as upper case.
c		301-399: Length (+300) of Character string returned
c		        padded with nulls
c			Lower case letters are converted to upper case.

c	(Integer,Real) IVAL
c		Address for returned number, character, array, or string
c		If a Character string is to be returned, call with
c		Equivalence'd Integer; otherwise, Fortran adds another
c		invisible string length to the calling parameter list
c		and messes up everything.
	Integer*2 IVAL(*)

	Include 'prompt.cmb'

c  Get current value of parameter from parameter string

	Call PGET (HSTRING,VNAME,ITYPE,IVAL)

c  Set up the prompt

	Call TPROMPT (PROMPT,VNAME)

c  Read new value from keyboard

	Call ENTER (PRMPT,ITYPE,IVAL)

c  Put value into parameter string

	Call PPUT (HSTRING,VNAME,ITYPE,IVAL)

	PRMPT = '          '
	If (LPN .lt. 16) PRMPT(LPN+1:LPN+1) = '$'

	Return
	End


c---  Read Integer*4 value for variable VNAME from terminal

	Subroutine TGETI4 (HSTRING,PROMPT,VNAME,IVAL)

	Character*(*) HSTRING
	Character*(*) PROMPT
	Character*(*) VNAME
	Integer*4 IVAL

	Include 'prompt.cmb'

	Call PGETI4 (HSTRING,VNAME,IVAL)
	Call TPROMPT (PROMPT,VNAME)
	Call ENTERI4 (PRMPT,IVAL)
	Call PPUTI4 (HSTRING,VNAME,IVAL)
	PRMPT = '          '
	If (LPN .lt. 16) PRMPT(LPN+1:LPN+1) = '$'

	Return
	End


c---  Read Integer*2 value for variable VNAME from terminal

	Subroutine TGETI2 (HSTRING,PROMPT,VNAME,IVAL)

	Character*(*) HSTRING
	Character*(*) PROMPT
	Character*(*) VNAME
	Integer*2 IVAL

	Include 'prompt.cmb'

	Call PGETI2 (HSTRING,VNAME,IVAL)
	Call TPROMPT (PROMPT,VNAME)
	Call ENTERI2 (PRMPT,IVAL)
	Call PPUTI2 (HSTRING,VNAME,IVAL)
	PRMPT = '          '
	If (LPN .lt. 16) PRMPT(LPN+1:LPN+1) = '$'

	Return
	End


c---  Read Real*4 value for variable VNAME from terminal

	Subroutine TGETR4 (HSTRING,PROMPT,VNAME,RVAL)

	Character*(*) HSTRING
	Character*(*) PROMPT
	Character*(*) VNAME
	Real*4 RVAL

	Include 'prompt.cmb'

	Call PGETR4 (HSTRING,VNAME,RVAL)
	Call TPROMPT (PROMPT,VNAME)
	Call ENTERR4 (PRMPT,RVAL)
	Call PPUTR4 (HSTRING,VNAME,RVAL)
	PRMPT = '          '
	If (LPN .lt. 16) PRMPT(LPN+1:LPN+1) = '$'

	Return
	End


c---  Read Real*8 value for variable VNAME from terminal

	Subroutine TGETR8 (HSTRING,PROMPT,VNAME,DVAL)

	Character*(*) HSTRING
	Character*(*) PROMPT
	Character*(*) VNAME
	Real*8 DVAL

	Include 'prompt.cmb'

	Call PGETR8 (HSTRING,VNAME,DVAL)
	Call TPROMPT (PROMPT,VNAME)
	Call ENTERR8 (PRMPT,DVAL)
	Call PPUTR8 (HSTRING,VNAME,DVAL)
	PRMPT = '          '
	If (LPN .lt. 16) PRMPT(LPN+1:LPN+1) = '$'

	Return
	End


c---  Read Logical value for variable VNAME from terminal

	Subroutine TGETL (HSTRING,PROMPT,VNAME,LVAL)

	Character*(*) HSTRING
	Character*(*) PROMPT
	Character*(*) VNAME
	Logical LVAL

	Include 'prompt.cmb'

	Call PGETL (HSTRING,VNAME,LVAL)
	Call TPROMPT (PROMPT,VNAME)
	Call ENTERL (PRMPT,LVAL)
	Call PPUTL (HSTRING,VNAME,LVAL)
	PRMPT = '          '
	If (LPN .lt. 16) PRMPT(LPN+1:LPN+1) = '$'

	Return
	End


	Subroutine TGETA (HSTRING,PROMPT,ITYPE,VNAME)

c  Get current value of parameter VNAME from parameter string HSTRING,
c  receive new value from keyboard using PROMPT and
c  enter "<VNAME> = <IVAL>" into parameter string HSTRING.
c  The parameter does not need to be declared in the calling program.

	Character*(*) HSTRING
	Character*(*) PROMPT
	Integer*4 ITYPE
	Character*(*) VNAME

	Integer*4 ITEMP
	Integer*2 HTEMP
	Logical LTEMP
	Real*4 RTEMP
	Real*8 DTEMP
c		Address for returned number
	Character CTEMP*99,CT*1
c		Address for returned character string

	Integer*4 LPRMT,LP,IVAL(8)
	Include 'header.cmb'
	Include 'prompt.cmb'

c  For character data

	If (ITYPE .gt. 301) Then
	    Call TGETC (HSTRING,PROMPT,VNAME,CTEMP)
	Elseif (ITYPE .gt. 300) Then
	    Call TGETC (HSTRING,PROMPT,VNAME,CT)
	    CTEMP = CT
	Elseif (ITYPE .gt. 101) Then
	    Call TGETS (HSTRING,PROMPT,VNAME,CTEMP)
	Elseif (ITYPE .gt. 100) Then
	    Call TGETS (HSTRING,PROMPT,VNAME,CT)
	    CTEMP = CT
	Elseif (ITYPE .gt. 1) Then
	    Call TGETC (HSTRING,PROMPT,VNAME,CTEMP)
	Elseif (ITYPE .ge. -1) Then
	    Call TGETC (HSTRING,PROMPT,VNAME,CT)
	    CTEMP = CT

c  For numerical data

	Elseif (ITYPE .eq. -12) Then
	    Call TGETI2 (HSTRING,PROMPT,VNAME,HTEMP)
	Elseif (ITYPE .eq. -14) Then
	    Call TGETI4 (HSTRING,PROMPT,VNAME,ITEMP)
	Elseif (ITYPE .eq. -24) Then
	    Call TGETR4 (HSTRING,PROMPT,VNAME,RTEMP)
	Elseif (ITYPE .eq. -28) Then
	    Call TGETR8 (HSTRING,PROMPT,VNAME,DTEMP)

c  For logical data

	Elseif (ITYPE .eq. -34) Then
	    Call TGETL (HSTRING,PROMPT,VNAME,LTEMP)
	Elseif (ITYPE .eq. -44) Then
	    Call TGETL (HSTRING,PROMPT,VNAME,LTEMP)

c  For other data

	Else
	    Call VLEN (PROMPT,LPRMT)
	    PRMPT(LPN+1:LPN+LPRMT) = PROMPT(1:LPRMT)
	    LP = LPN + LPRMT + 1
	    PRMPT(LP:LP) = Char(0)
	    Call PGET (HSTRING,VNAME,ITYPE,IVAL)
	    Call ENTER (PRMPT,ITYPE,IVAL)
	    Call PPUT (HSTRING,VNAME,ITYPE,IVAL)
	    PRMPT = '          '
	    If (LPN .lt. 16) PRMPT(LPN+1:LPN+1) = '$'
	Endif

	Return
	End


c-- Read numeric value from terminal only; don't modifiy string

	Subroutine TGETN (PROMPT,ITYPE,IVAL)

	Character*(*) PROMPT
	Integer*4 ITYPE
	Integer*2 IVAL(*)
c			(Integer,Real) IVAL

	Integer*4 LPRMT,LP
	Include 'prompt.cmb'

c  Set up the prompt

	Call VLEN (PROMPT,LPRMT)
	PRMPT(LPN+1:LPN+LPRMT) = PROMPT(1:LPRMT)
	LP = LPN + LPRMT + 1
	PRMPT(LP:LP) = Char(0)

c  Read new value from keyboard

	Call ENTER (PRMPT,ITYPE,IVAL)

	PRMPT = '          '
	If (LPN .lt. 16) PRMPT(LPN+1:LPN+1) = '$'

	Return
	End


c-- Read character value from terminal only; don't modifiy string

	Subroutine TGETNC (PROMPT,ITYPE,CVAL)

	Character*(*) PROMPT
	Integer*4 ITYPE
	Character*(*) CVAL

	Integer*4 LPRMT,LP
	Include 'prompt.cmb'

c  Set up the prompt

	Call VLEN (PROMPT,LPRMT)
	PRMPT(LPN+1:LPN+LPRMT) = PROMPT(1:LPRMT)
	LP = LPN + LPRMT + 1
	PRMPT(LP:LP) = Char(0)

c  Read new value from keyboard

	Call ENTERC (PRMPT,ITYPE,CVAL)

	PRMPT = '          '
	If (LPN .lt. 16) PRMPT(LPN+1:LPN+1) = '$'

	Return
	End


c---  Read null-terminated character string for variable VNAME from terminal
c     Change all lower case characters to upper case

	Subroutine TGETC (HSTRING,PROMPT,VNAME,CVAL)

	Character*(*) HSTRING
	Character*(*) PROMPT
	Character*(*) VNAME
	Character*(*) CVAL
c			Character string containing the value for variable
c			VNAME.  Trailing and leading blanks are removed.

	Character*1 NULL
	Integer*4 ITYPE,LCVAL,LVAL

	Integer*4 L
	Include 'prompt.cmb'
	Include 'header.cmb'

	Call PGETC (HSTRING,VNAME,CVAL)

	LCVAL = Len (CVAL)
	Call VLEN (CVAL,LVAL)
	If (LVAL .lt. LCVAL) Then
	    NULL = Char (0)
	    Do 50 L = LVAL+1, LCVAL
50		CVAL(L:L) = NULL
	    Endif
	ITYPE = LCVAL + 300

c  Set up the prompt

	Call TPROMPT (PROMPT,VNAME)

c  Read new string from keyboard

	Call ENTERC (PRMPT,ITYPE,CVAL)

c  Put string into parameter string

	Call PPUTS (HSTRING,VNAME,CVAL)

	PRMPT = '          '
	If (LPN .lt. 16) PRMPT(LPN+1:LPN+1) = '$'

	Return
	End


c---  Read Character null-terminated string for variable VNAME from terminal

	Subroutine TGETS (HSTRING,PROMPT,VNAME,CVAL)

	Character*(*) HSTRING
	Character*(*) PROMPT
	Character*(*) VNAME
	Character*(*) CVAL
c			Character string containing the value for variable
c			VNAME.  Trailing and leading blanks are removed.

	Integer*4 ITYPE
	Integer*4 LCVAL
	Include 'prompt.cmb'
	Include 'header.cmb'

	Call PGETC (HSTRING,VNAME,CVAL)

	LCVAL = Len (CVAL)
c	Call VLEN (CVAL,LVAL)
c	If (VERBOSE) Write(*,*) 'PGETS:  string length is',LVAL,'/',LCVAL
c	If (LVAL .lt. LCVAL) Then
c	    Do 50 L = LVAL+1, LCVAL
c50		CVAL(L:L) = Char (0)
c	    Endif
	ITYPE = LCVAL + 100

c  Set up the prompt

	Call TPROMPT (PROMPT,VNAME)

c  Read new string from keyboard

	Call ENTERC (PRMPT,ITYPE,CVAL)

c  Put string into parameter string

	Call PPUTS (HSTRING,VNAME,CVAL)

	PRMPT = '          '
	If (LPN .lt. 16) PRMPT(LPN+1:LPN+1) = '$'

	Return
	End


c---  Read single character command from terminal

	Subroutine TGETCOM (PROMPT,CCOM)

	Character*1 CCOM
c			Character command returned
	Character*(*) PROMPT

	Integer*4 LPR
	Include 'prompt.cmb'

c  Read single menu selection from keyboard if prompt is null

	If (PROMPT(1:1) .eq. Char (0)) Then
	    Call ENTERC (PRGNAM,-1,CCOM)
	Else

c  Otherwise construct prompt from program name and PROMPT and read character

	    Call TPNAM
	    Call VLEN (PROMPT,LPR)
	    PRMPT(LPN+1:LPN+LPR) = PROMPT(1:LPR)
	    PRMPT(LPN+LPR+1:LPN+LPR+1) = Char(0)
	    Call ENTERC (PRMPT,-1,CCOM)
	Endif

	PRMPT = '          '
	If (LPN .lt. 16) PRMPT(LPN+1:LPN+1) = '$'

	Return
	End


c---  Read single character command from terminal

	Subroutine TGETCHAR (PROMPT,CCOM)

	Character*1 CCOM
c			Character returned
	Character*(*) PROMPT

	Integer*4 LPR
	Include 'prompt.cmb'

c  Set up prompt

	Call VLEN (PROMPT,LPR)
	PRMPT(LPN+1:LPN+LPR) = PROMPT(1:LPR)
	PRMPT(LPN+LPR+1:LPN+LPR+1) = Char(0)

c  Read character from keyboard

	Call ENTERC (PRMPT,301,CCOM)

	PRMPT = '          '
	If (LPN .lt. 16) PRMPT(LPN+1:LPN+1) = '$'

	Return
	End


c---  Set program name for prompt

	Subroutine TPROG (PROGNAME)

	Character*(*) PROGNAME
c				Program name to be used in prompt message

	Include 'prompt.cmb'

	PRGNAM = PROGNAME
	Call PADNULL (PRGNAM)
	Call VLEN (PRGNAM,LPN)
	LPN = LPN + 2
	PRMPT = '          '
	If (LPN .lt. 16) PRMPT(LPN+1:LPN+1) = '$'

	Return
	End


c---  Get program name

	Subroutine TPGET (PROGNAME)

	Character*(*) PROGNAME
c				Program name to be used in prompt message

	Include 'prompt.cmb'

	PROGNAME = PRGNAM
	Call PADNULL (PROGNAME)

	Return
	End


c---  Add program name to next prompt

	Subroutine TPNAM ()

	Include 'prompt.cmb'

	PRMPT(1:LPN-2) = PRGNAM
	PRMPT(LPN-1:LPN) = '? '

	Return
	End


c-- Write string preceded by command prompt

	Subroutine TPUT (STRING)

	Character*(*) STRING

	Integer*4 LSTR,LPRMPT

	Integer*4 LP
	Include 'prompt.cmb'

c  Set up the prompt

	Call VLEN (STRING,LSTR)
	LPRMPT = Len (PRMPT)
	If ((LPN+LSTR+1) .gt. LPRMPT) LSTR = LPRMPT - LPN - 1
	PRMPT(LPN+1:LPN+LSTR) = STRING(1:LSTR)
	LP = LPN + LSTR + 1
	PRMPT(LP:LP) = Char(0)

c  Write new string to terminal

	Call PSTRING (PRMPT)

	PRMPT = '          '
	If (LPN .lt. 16) PRMPT(LPN+1:LPN+1) = '$'

	Return
	End


	Subroutine TPROMPT (PROMPT,VNAME)

	Character*(*) PROMPT
c		Prompt to be printed at terminal
c		If PROMPT is null, use comment from parameter file.
c		If there is no comment, prompt with parameter name only.

	Character*(*) VNAME
c		Character string containing the name of the variable
c		to be returned.

	Integer*4 LP,LPRMT,LVNAME,LCOM

	Include 'prompt.cmb'
	Include 'header.cmb'

c  Set up the prompt

	Call VLEN (PROMPT,LPRMT)
	Call VLEN (PCOM,LCOM)
	Call VLEN (VNAME,LVNAME)
	If (LPRMT .gt. 0) Then
	    PRMPT(LPN+1:LPN+LPRMT) = PROMPT(1:LPRMT)
	    LP = LPN + LPRMT + 1
	    PRMPT(LP:LP) = Char(0)
	    Call PADNULL (PRMPT)
c	    If (VERBOSE) Write(*,*) 'Prompt',LPRMT,
c     1	    'Name',LVNAME,'Prompt',LP

c  If there is no prompt argument, use the parameter file comment

	Elseif (LCOM .gt. 0) Then
	    PRMPT(LPN+1:LPN+LCOM) = PCOM(1:LCOM)
	    LP = LPN + LCOM + 1
	    PRMPT(LP:LP) = Char(0)
	    Call PADNULL (PRMPT)
c	    If (VERBOSE) Write(*,*) 'Comment',LCOM,
c     1	    'Name',LVNAME,'Prompt',LP

c  If there is no comment field, prompt with parameter name

	Else
	    If (LVNAME .lt. 1) Then
		If (VERBOSE) Call PSTRING ('TGET: No variable name is present')
		PRMPT(LPN+1:LPN+17) = 'Mystery parameter'
		Call PADNULL (PRMPT)
	    Else
		PRMPT(LPN+1:LPN+LVNAME) = VNAME(1:LVNAME)
		LP = LPN + LVNAME + 1
		PRMPT(LP:LP) = Char(0)
		Call PADNULL (PRMPT)
	    Endif
	Endif

	Return
	End


c Jan 22 1988	Replace padstr call with padnull

c Sep 11 1992	Call tget* from tgeta for accuracy on RISC machines
c		Add tprompt to set prompt uniformly
c		Call pent* to match binary argument types
c Nov 17 1992	Add tpget to retrieve program name

c Jun 28 1993	Treat single-character responses specially in tgeta
c Jun 29 1993	Fix tgeta to get up to 99 characters instead of 20
c Jul  6 1993	Drop prompt diagnostic

c Apr  7 1994	Fix tpget

c Sep 14 1995	Set blank prompt more carefully
